@@ -66,6 +66,7 @@
 #include "hostType.h"
 #include "vm_atomic.h"
 #include "fileLock.h"
+#include "userlock.h"
 
 #include "unicodeOperations.h"
 
@@ -284,12 +285,38 @@ File_UnlinkNoFollow(ConstUnicode pathName)  // IN:
 }
 
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * File_CreateDirectoryEx --
+ *
+ *      Creates the specified directory with the specified permissions.
+ *
+ * Results:
+ *      True if the directory is successfully created, false otherwise.
+ *
+ * Side effects:
+ *      Creates the directory on disk.
+ *
+ *----------------------------------------------------------------------
+ */
+
+Bool
+File_CreateDirectoryEx(ConstUnicode pathName,  // IN:
+                       int mask)               // IN:
+{
+   int err = FileCreateDirectory(pathName, mask);
+
+   return err == 0;
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
  * File_CreateDirectory --
  *
- *      Creates the specified directory.
+ *      Creates the specified directory with 0777 permissions.
  *
  * Results:
  *      True if the directory is successfully created, false otherwise.
@@ -303,9 +330,7 @@ File_UnlinkNoFollow(ConstUnicode pathName)  // IN:
 Bool
 File_CreateDirectory(ConstUnicode pathName)  // IN:
 {
-   int err = FileCreateDirectory(pathName, 0777);
-
-   return err == 0;
+   return File_CreateDirectoryEx(pathName, 0777);
 }
 
 
@@ -1421,6 +1446,33 @@ File_MoveTree(ConstUnicode srcName,   // IN:
          }
       }
 
+#if !defined(__FreeBSD__) && !defined(sun)
+      /*
+       * File_GetFreeSpace is not defined for FreeBSD
+       */
+      if (createdDir) {
+         /*
+          * Check for free space on destination filesystem.
+          * We only check for free space if the destination directory
+          * did not exist. In this case, we will not be overwriting any existing
+          * paths, so we need as much space as srcName.
+          */
+         int64 srcSize;
+         int64 freeSpace;
+         srcSize = File_GetSizeEx(srcName);
+         freeSpace = File_GetFreeSpace(dstName, TRUE);
+         if (freeSpace < srcSize) {
+            Unicode spaceStr = Msg_FormatSizeInBytes(srcSize);
+            Msg_Append(MSGID(File.MoveTree.dst.insufficientSpace)
+                  "There is not enough space in the file system to "
+                  "move the directory tree. Free %s and try again.",
+                  spaceStr);
+            free(spaceStr);
+            return FALSE;
+         }
+      }
+#endif
+
       if (File_CopyTree(srcName, dstName, overwriteExisting, FALSE)) {
          ret = TRUE;
 
@@ -1646,9 +1698,10 @@ File_GetSizeByPath(ConstUnicode pathName)  // IN:
 /*
  *-----------------------------------------------------------------------------
  *
- * File_CreateDirectoryHierarchy --
+ * File_CreateDirectoryHierarchyEx --
  *
  *      Create a directory including any parents that don't already exist.
+ *      All the created directories are tagged with the specified permission.
  *      Returns the topmost directory which was created, to allow calling code
  *      to remove it after in case later operations fail.
  *
@@ -1669,8 +1722,9 @@ File_GetSizeByPath(ConstUnicode pathName)  // IN:
  */
 
 Bool
-File_CreateDirectoryHierarchy(ConstUnicode pathName,   // IN:
-                              Unicode *topmostCreated) // OUT:
+File_CreateDirectoryHierarchyEx(ConstUnicode pathName,   // IN:
+                                int mask,                // IN
+                                Unicode *topmostCreated) // OUT:
 {
    Unicode volume;
    UnicodeIndex index;
@@ -1719,7 +1773,7 @@ File_CreateDirectoryHierarchy(ConstUnicode pathName,   // IN:
       if (File_IsDirectory(temp)) {
          failed = FALSE;
       } else {
-         failed = !File_CreateDirectory(temp);
+         failed = !File_CreateDirectoryEx(temp, mask);
          if (!failed && topmostCreated != NULL && *topmostCreated == NULL) {
             *topmostCreated = temp;
             temp = NULL;
@@ -1742,6 +1796,42 @@ File_CreateDirectoryHierarchy(ConstUnicode pathName,   // IN:
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * File_CreateDirectoryHierarchy --
+ *
+ *      Create a directory including any parents that don't already exist.
+ *      All the created directories are tagged with 0777 permissions.
+ *      Returns the topmost directory which was created, to allow calling code
+ *      to remove it after in case later operations fail.
+ *
+ * Results:
+ *      TRUE on success, FALSE on failure.
+ *
+ *      If topmostCreated is not NULL, it returns the result of the hierarchy
+ *      creation. If no directory was created, *topmostCreated is set to NULL.
+ *      Otherwise *topmostCreated is set to the topmost directory which was
+ *      created. *topmostCreated is set even in case of failure.
+ *
+ *      The caller most Unicode_Free the resulting string.
+ *
+ * Side effects:
+ *      Only the obvious.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+Bool
+File_CreateDirectoryHierarchy(ConstUnicode pathName,   // IN:
+                              Unicode *topmostCreated) // OUT:
+{
+   return File_CreateDirectoryHierarchyEx(pathName,
+                                          0777,
+                                          topmostCreated);
+}
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -2050,9 +2140,6 @@ File_ExpandAndCheckDir(const char *dirName)  // IN:
  *
  *      Return a random number in the range of 0 and 2^32-1.
  *
- *      This isn't thread safe but it's more than good enough for the
- *      purposes required of it.
- *
  * Results:
  *      Random number is returned.
  *
@@ -2065,38 +2152,35 @@ File_ExpandAndCheckDir(const char *dirName)  // IN:
 uint32
 FileSimpleRandom(void)
 {
-   static Atomic_Ptr atomic; /* Implicitly initialized to NULL. --mbellon */
-   rqContext *context;
+   static Atomic_Ptr lckStorage;
+   static rqContext *context = NULL;
+   uint32 result;
+   MXUserExclLock *lck = MXUser_CreateSingletonExclLock(&lckStorage,
+                                                        "fileSimpleRandomLock",
+                                                        RANK_LEAF);
 
-   context = Atomic_ReadPtr(&atomic);
+   ASSERT_NOT_IMPLEMENTED(lck != NULL);
+
+   MXUser_AcquireExclLock(lck);
 
    if (UNLIKELY(context == NULL)) {
-      rqContext *newContext;
       uint32 value;
 
-      /*
-       * Threads will hash up this RNG - this isn't officially thread safe
-       * which is just fine - but ensure that different processes have
-       * different answer streams.
-       */
-
 #if defined(_WIN32)
       value = GetCurrentProcessId();
 #else
       value = getpid();
 #endif
 
-      newContext = Random_QuickSeed(value);
-
-      if (Atomic_ReadIfEqualWritePtr(&atomic, NULL, (void *) newContext)) {
-         free(newContext);
-      }
-
-      context = Atomic_ReadPtr(&atomic);
+      context = Random_QuickSeed(value);
       ASSERT(context);
    }
 
-   return Random_Quick(context);
+   result = Random_Quick(context);
+
+   MXUser_ReleaseExclLock(lck);
+
+   return result;
 }
 
 
@@ -2178,6 +2262,10 @@ FileRotateByRename(const char *fileName,  // IN: full path to file
    int i;
    int result;
 
+   if (newFileName != NULL) {
+      *newFileName = NULL;
+   }
+
    for (i = n; i >= 0; i--) {
       src = (i == 0) ? (char *) fileName :
                        Str_SafeAsprintf(NULL, "%s-%d%s", baseName, i - 1, ext);
@@ -2202,8 +2290,8 @@ FileRotateByRename(const char *fileName,  // IN: full path to file
          }
       }
 
-      if ((src == fileName) && (newFileName != NULL)) {
-         *newFileName = result == -1 ? NULL : strdup(dst);
+      if ((src == fileName) && (newFileName != NULL) && (result == 0)) {
+         *newFileName = Util_SafeStrdup(dst);
       }
 
       ASSERT(dst != fileName);
@@ -2276,6 +2364,10 @@ FileRotateByRenumber(const char *filePath,       // IN: full path to file
    uint32 *fileNumbers = NULL;
    int result;
 
+   if (newFilePath != NULL) {
+      *newFilePath = NULL;
+   }
+
    fullPathNoExt = File_FullPath(filePathNoExt);
    if (fullPathNoExt == NULL) {
       Log(LGPFX" %s: failed to get full path for '%s'.\n", __FUNCTION__,
@@ -2339,7 +2431,6 @@ FileRotateByRenumber(const char *filePath,       // IN: full path to file
 
    if (newFilePath != NULL) {
       if (result == -1) {
-         *newFilePath = NULL;
          free(tmp);
       } else {
          *newFilePath = tmp;
