@@ -104,11 +104,19 @@ Bool FileMacos_IsOnSparseDmg(int fd);
 Bool FileMacos_IsSliceDevice(char const *bsdDev);
 
 char *FileMacos_DiskDevToUserFriendlyName(char const *bsdDiskDev);
+char *FileMacos_DiskDevToVolumeName(char const *bsdDiskDev);
 
 char *FileMacos_DiskDeviceToUniqueID(char const *bsdPath);
 char *FileMacos_UniqueIDToDiskDevice(char const *identifier);
-
+Bool FileMacOS_MakeSecureLibraryCopies(const char   *inDir,
+                                       const char  **dylibName,
+                                       unsigned      numDylibs,
+                                       char        **outDir);
 #elif defined VMX86_SERVER
+struct FS_PartitionListResult;
+
+int File_GetVMFSAttributes(ConstUnicode pathName,
+                           struct FS_PartitionListResult **fsAttrs);
 int File_GetVMFSVersion(ConstUnicode pathName,
                         uint32 *versionNum);
 int File_GetVMFSBlockSize(ConstUnicode pathName,
@@ -126,6 +134,8 @@ Bool File_SupportsZeroedThick(ConstUnicode pathName);
 
 Bool File_SupportsMultiWriter(ConstUnicode pathName);
 
+Bool File_SupportsMandatoryLock(ConstUnicode pathName);
+
 Bool File_Exists(ConstUnicode pathName);
 
 int File_Unlink(ConstUnicode pathName);
@@ -151,11 +161,19 @@ Unicode File_PathJoin(ConstUnicode dirName,
                       ConstUnicode baseName);
 
 Bool File_CreateDirectory(ConstUnicode pathName);
+
+Bool File_CreateDirectoryEx(ConstUnicode pathName, int mask);
+
 Bool File_EnsureDirectory(ConstUnicode pathName);
 
 Bool File_DeleteEmptyDirectory(ConstUnicode pathName);
 
-Bool File_CreateDirectoryHierarchy(ConstUnicode pathName);
+Bool File_CreateDirectoryHierarchy(ConstUnicode pathName,
+                                   Unicode *topmostCreated);
+
+Bool File_CreateDirectoryHierarchyEx(ConstUnicode pathName,
+                                     int mask,
+                                     Unicode *topmostCreated);
 
 Bool File_DeleteDirectoryTree(ConstUnicode pathName);
 
@@ -233,6 +251,13 @@ Bool File_SetFilePermissions(ConstUnicode pathName,
 Bool File_SupportsFileSize(ConstUnicode pathName,
                            uint64 fileSize);
 
+Bool File_GetMaxFileSize(ConstUnicode pathName,
+                         uint64 *maxFileSize);
+
+#ifdef VMX86_SERVER
+Bool File_Is2TiBEnabled(void);
+#endif
+
 Bool File_SupportsLargeFiles(ConstUnicode pathName);
 
 char *File_MapPathPrefix(const char *oldPath,
@@ -276,6 +301,13 @@ Bool File_CopyTree(ConstUnicode srcName,
 Bool File_Replace(ConstUnicode oldFile,
                   ConstUnicode newFile);
 
+int File_Rename(ConstUnicode oldFile,
+                ConstUnicode newFile);
+
+int File_RenameRetry(ConstUnicode oldFile,
+                     ConstUnicode newFile,
+                     uint32 msecMaxWaitTime);
+
 Bool File_Move(ConstUnicode oldFile,
                ConstUnicode newFile,
                Bool *asRename);
@@ -285,6 +317,13 @@ void File_Rotate(const char *pathName,
                  Bool noRename,
                  char **newFileName);
 
+int File_GetFSMountInfo(ConstUnicode pathName,
+                        char **fsType,
+                        uint32 *version,
+                        char **remoteIP,
+                        char **remoteMountPoint,
+                        char **localMountPoint);
+
 /* Get size only for regular file. */
 int64 File_GetSize(ConstUnicode pathName);
 
