@@ -87,7 +87,7 @@
    LOGLEVEL_VAR(pci_hyper), \
    LOGLEVEL_VAR(pcibridge), \
    LOGLEVEL_VAR(vide), \
-   LOGLEVEL_VAR(ideCdrom), \
+   LOGLEVEL_VAR(atapiCdrom), \
    LOGLEVEL_VAR(hostonly), \
    LOGLEVEL_VAR(oprom), \
    LOGLEVEL_VAR(http), \
@@ -133,6 +133,7 @@
    LOGLEVEL_VAR(mksGLFBO), \
    LOGLEVEL_VAR(mksGLShader), \
    LOGLEVEL_VAR(mksGLState), \
+   LOGLEVEL_VAR(mksGLWindow), \
    LOGLEVEL_VAR(vdpPlugin), \
    \
    /* user/sound */ \
@@ -166,6 +167,7 @@
    LOGLEVEL_VAR(buslogic), \
    LOGLEVEL_VAR(lsilogic), \
    LOGLEVEL_VAR(pvscsi), \
+   LOGLEVEL_VAR(ahci), \
    LOGLEVEL_VAR(diskVmnix), \
    LOGLEVEL_VAR(hbaCommon), \
    LOGLEVEL_VAR(backdoor), \
@@ -262,6 +264,11 @@
    LOGLEVEL_VAR(grainTrack), \
    LOGLEVEL_VAR(shim3D), \
    LOGLEVEL_VAR(crc32), \
+   LOGLEVEL_VAR(vmkmgmtlib), \
+   LOGLEVEL_VAR(vflash), \
+   LOGLEVEL_VAR(vva),  /* apps/rde/vva */ \
+   LOGLEVEL_VAR(ftConfig), /*lib/ftConfig */ \
+   LOGLEVEL_VAR(vmname),  /* lib/vmname */ \
    /* end of list */
 
 LOGLEVEL_EXTENSION_DECLARE(LOGLEVEL_USER);
