@@ -29,22 +29,25 @@
 
 #include "vm_basic_types.h"
 
-
-Bool
-Random_Crypto(unsigned int size, // IN
-              void *buffer);     // OUT
+Bool Random_Crypto(size_t size,
+                   void *buffer);
 
 /*
- * High quality random number generator.
+ * High quality - research grade - random number generator.
+ *
+ * Despite its apparent complexity this RNG is extremely fast.
  */
 
-void *
-Random_QuickSeed(uint32 seed); // IN
+typedef struct rqContext rqContext;
 
-uint32
-Random_Quick(void *context);
+rqContext *Random_QuickSeed(uint32 seed);
 
-int
-FastRand(int seed);
+uint32 Random_Quick(rqContext *context);
+
+/*
+ * Simple multiplicative congruential RNG.
+ */
+
+int Random_Simple(int seed);
 
 #endif /* __RANDOM_H__ */
