@@ -34,26 +34,18 @@
 #endif
 #include "unicode.h"
 
+uint64 System_GetTimeMonotonic(void);
 uint64 System_Uptime(void);
-Bool System_GetCurrentTime(int64 *secs, int64 *usecs);
-Bool System_AddToCurrentTime(int64 deltaSecs, int64 deltaUsecs);
 Unicode System_GetTimeAsString(void);
-Bool System_EnableTimeSlew(int64 delta, int64 timeSyncPeriod);
-Bool System_DisableTimeSlew(void);
-Bool System_IsTimeSlewEnabled(void);
-Bool System_IsACPI(void);
 void System_Shutdown(Bool reboot);
-Bool System_IsUserAdmin(void);
+Bool System_GetNodeName(size_t outBufSize, char *outBuf);
 
 char *System_GetEnv(Bool global, const char *valueName);
 int System_SetEnv(Bool global, const char *valueName, const char *value);
 
-#ifdef _WIN32
-typedef enum AeroStateCommand {
-   AeroCommand_Enable,
-   AeroCommand_Disable,
-} AeroStateCommand;
+Bool System_IsUserAdmin(void);
 
+#ifdef _WIN32
 /*
  * Representation of monitors gathered by System_GetMonitors.
  */
@@ -75,10 +67,9 @@ typedef struct MonListNode {
  */
 #define VM_SERVICE_STATE_UNKNOWN 0xffffffff
 
-BOOL System_SetProcessPrivilege(LPCTSTR lpszPrivilege, Bool bEnablePrivilege);
+BOOL System_SetProcessPrivilege(wchar_t *privName, Bool enable);
 int32 System_GetSPVersion(void);
 Bool System_IsLoginScreenActive(void);
-Bool System_IsProcessElevated(void);
 Bool System_IsScreenSaverActive(void);
 Bool System_IsScreenSaverRunning(void);
 Bool System_IsSecureDesktopActive(void);
@@ -86,7 +77,7 @@ Bool System_DisableAndKillScreenSaver(void);
 DWORD System_GetServiceState(LPCWSTR szServiceName);
 DblLnkLst_Links *System_GetMonitors();
 void System_SetFocusedWindow(HWND windowToFocus, Bool force);
-Bool System_SetAeroState(AeroStateCommand command, AeroStateCommand *oldState);
+Bool System_EnableDesktopComposition(BOOL enabled);
 #endif
 
 
@@ -96,12 +87,9 @@ Bool System_SetAeroState(AeroStateCommand command, AeroStateCommand *oldState);
  *        then acted upon and translates to a -DPOSIX_LIKE_ENVIRONMENT
  *        preprocessor option.
  */
-#if !defined(_WIN32) && !defined(N_PLAT_NLM)
-Bool System_WritePidFile(const char *fileName, pid_t pid);
+#if !defined(_WIN32)
 const char **System_GetNativeEnviron(const char **compatEnviron);
 void System_FreeNativeEnviron(const char **nativeEnviron);
-int System_UnsetEnv(const char *variableName);
-char *System_SetLDPath(const char *path, const Bool native);
 #endif
 
 #endif /* __SYSTEM_H__ */
