@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 1998 VMware, Inc. All rights reserved.
+ * Copyright (C) 1998-2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -41,59 +41,60 @@
    #endif
 #else
    #include <sys/types.h>
+   #include "errno.h"
 #endif
-
 #include "vm_assert.h"
 #include "unicodeTypes.h"
 
 
 /*
- * Define the Util_ThreadID type.
+ * Define the Util_ThreadID type, and assorted standard bits.
  */
-#if defined(__APPLE__) || defined(__FreeBSD__)
-#include <pthread.h>
-typedef pthread_t Util_ThreadID;
-#elif defined(_WIN32)
-typedef DWORD Util_ThreadID;
-#else // Linux et al
-#include <unistd.h>
-typedef pid_t Util_ThreadID;
+#if defined(_WIN32)
+   typedef DWORD Util_ThreadID;
+#else
+   #include <unistd.h>
+   #if defined(__APPLE__) || defined(__FreeBSD__)
+      #include <pthread.h>
+      typedef pthread_t Util_ThreadID;
+   #else // Linux et al
+      typedef pid_t Util_ThreadID;
+   #endif
 #endif
 
-
-EXTERN uint32 CRC_Compute(const uint8 *buf, int len);
-EXTERN uint32 Util_Checksum32(const uint32 *buf, int len);
-EXTERN uint32 Util_Checksum(const uint8 *buf, int len);
-EXTERN uint32 Util_Checksumv(void *iov, int numEntries);
-EXTERN uint32 Util_HashString(const char *str);
-EXTERN Unicode Util_ExpandString(ConstUnicode fileName);
-EXTERN void Util_ExitThread(int);
-EXTERN NORETURN void Util_ExitProcessAbruptly(int);
-EXTERN int Util_HasAdminPriv(void);
+uint32 CRC_Compute(const uint8 *buf, int len);
+uint32 Util_Checksum32(const uint32 *buf, int len);
+uint32 Util_Checksum(const uint8 *buf, int len);
+uint32 Util_Checksumv(void *iov, int numEntries);
+uint32 Util_HashString(const char *str);
+Unicode Util_ExpandString(ConstUnicode fileName);
+void Util_ExitThread(int);
+NORETURN void Util_ExitProcessAbruptly(int);
+int Util_HasAdminPriv(void);
 #if defined _WIN32 && defined USERLEVEL
-EXTERN int Util_TokenHasAdminPriv(HANDLE token);
-EXTERN int Util_TokenHasInteractPriv(HANDLE token);
+int Util_TokenHasAdminPriv(HANDLE token);
+int Util_TokenHasInteractPriv(HANDLE token);
 #endif
-EXTERN Bool Util_Data2Buffer(char *buf, size_t bufSize, const void *data0,
-                             size_t dataSize);
-EXTERN char *Util_GetCanonicalPath(const char *path);
+Bool Util_Data2Buffer(char *buf, size_t bufSize, const void *data0,
+                      size_t dataSize);
+char *Util_GetCanonicalPath(const char *path);
 #ifdef _WIN32
-EXTERN char *Util_CompatGetCanonicalPath(const char *path);
-EXTERN char *Util_GetCanonicalPathForHash(const char *path);
-EXTERN char *Util_CompatGetLowerCaseCanonicalPath(const char* path);
+char *Util_CompatGetCanonicalPath(const char *path);
+char *Util_GetCanonicalPathForHash(const char *path);
+char *Util_CompatGetLowerCaseCanonicalPath(const char* path);
 #endif
-EXTERN int Util_BumpNoFds(uint32 *cur, uint32 *wanted);
-EXTERN Bool Util_CanonicalPathsIdentical(const char *path1, const char *path2);
-EXTERN Bool Util_IsAbsolutePath(const char *path);
-EXTERN unsigned Util_GetPrime(unsigned n0);
-EXTERN Util_ThreadID Util_GetCurrentThreadId(void);
+int Util_BumpNoFds(uint32 *cur, uint32 *wanted);
+Bool Util_CanonicalPathsIdentical(const char *path1, const char *path2);
+Bool Util_IsAbsolutePath(const char *path);
+unsigned Util_GetPrime(unsigned n0);
+Util_ThreadID Util_GetCurrentThreadId(void);
 
-EXTERN char *Util_DeriveFileName(const char *source,
-                                 const char *name,
-                                 const char *ext);
+char *Util_DeriveFileName(const char *source,
+                          const char *name,
+                          const char *ext);
 
-EXTERN char *Util_CombineStrings(char **sources, int count);
-EXTERN char **Util_SeparateStrings(char *source, int *count);
+char *Util_CombineStrings(char **sources, int count);
+char **Util_SeparateStrings(char *source, int *count);
 
 typedef struct UtilSingleUseResource UtilSingleUseResource;
 UtilSingleUseResource *Util_SingleUseAcquire(const char *name);
@@ -101,7 +102,7 @@ void Util_SingleUseRelease(UtilSingleUseResource *res);
 
 
 #if defined(__linux__) || defined(__FreeBSD__) || defined(sun)
-EXTERN Bool Util_GetProcessName(pid_t pid, char *bufOut, size_t bufOutSize);
+Bool Util_GetProcessName(pid_t pid, char *bufOut, size_t bufOutSize);
 #endif
 
 // backtrace functions and utilities
@@ -121,6 +122,11 @@ void Util_BacktraceWithFunc(int bugNr,
 void Util_BacktraceToBuffer(uintptr_t *basePtr,
                             uintptr_t *buffer, int len);
 
+// sleep functions
+
+void Util_Usleep(long usec);
+void Util_Sleep(unsigned int sec);
+
 int Util_CompareDotted(const char *s1, const char *s2);
 
 /*
@@ -146,8 +152,12 @@ Bool Util_QueryCStResidency(uint32 *numCpus, uint32 *numCStates,
 /*
  * In util_shared.h
  */
-EXTERN Bool Util_Throttle(uint32 count);
-EXTERN uint32 Util_FastRand(uint32 seed);
+Bool Util_Throttle(uint32 count);
+uint32 Util_FastRand(uint32 seed);
+
+
+// Not thread safe!
+void Util_OverrideHomeDir(const char *path);
 
 
 /*
@@ -250,8 +260,8 @@ Util_BufferIsEmpty(void const *base,  // IN:
 }
 
 
-EXTERN Bool Util_MakeSureDirExistsAndAccessible(char const *path,
-                                                unsigned int mode);
+Bool Util_MakeSureDirExistsAndAccessible(char const *path,
+                                         unsigned int mode);
 
 #if _WIN32
 #   define DIRSEPS	      "\\"
@@ -302,50 +312,98 @@ EXTERN Bool Util_MakeSureDirExistsAndAccessible(char const *path,
  *--------------------------------------------------------------------------
  */
 
-EXTERN void *Util_SafeInternalMalloc(int bugNumber, size_t size,
-                                     const char *file, int lineno);
+void *UtilSafeMalloc0(size_t size);
+void *UtilSafeMalloc1(size_t size,
+                      int bugNumber, const char *file, int lineno);
 
-EXTERN void *Util_SafeInternalRealloc(int bugNumber, void *ptr, size_t size,
-                                      const char *file, int lineno);
+void *UtilSafeRealloc0(void *ptr, size_t size);
+void *UtilSafeRealloc1(void *ptr, size_t size,
+                      int bugNumber, const char *file, int lineno);
 
-EXTERN void *Util_SafeInternalCalloc(int bugNumber, size_t nmemb,
-                                     size_t size, const char *file, int lineno);
+void *UtilSafeCalloc0(size_t nmemb, size_t size);
+void *UtilSafeCalloc1(size_t nmemb, size_t size,
+                      int bugNumber, const char *file, int lineno);
 
-EXTERN char *Util_SafeInternalStrdup(int bugNumber, const char *s,
-                                     const char *file, int lineno);
+char *UtilSafeStrdup0(const char *s);
+char *UtilSafeStrdup1(const char *s,
+                      int bugNumber, const char *file, int lineno);
 
-EXTERN char *Util_SafeInternalStrndup(int bugNumber, const char *s, size_t n,
-                                      const char *file, int lineno);
+char *UtilSafeStrndup0(const char *s, size_t n);
+char *UtilSafeStrndup1(const char *s, size_t n,
+                      int bugNumber, const char *file, int lineno);
+
+/* 
+ * Debug builds carry extra arguments into the allocation functions for
+ * better error reporting. Non-debug builds don't pay this extra overhead.
+ */
+#ifdef VMX86_DEBUG
 
 #define Util_SafeMalloc(_size) \
-   Util_SafeInternalMalloc(-1, (_size), __FILE__, __LINE__)
+   UtilSafeMalloc1((_size), -1, __FILE__, __LINE__)
 
 #define Util_SafeMallocBug(_bugNr, _size) \
-   Util_SafeInternalMalloc((_bugNr), (_size), __FILE__, __LINE__)
+   UtilSafeMalloc1((_size),(_bugNr), __FILE__, __LINE__)
 
 #define Util_SafeRealloc(_ptr, _size) \
-   Util_SafeInternalRealloc(-1, (_ptr), (_size), __FILE__, __LINE__)
+   UtilSafeRealloc1((_ptr), (_size), -1, __FILE__, __LINE__)
 
 #define Util_SafeReallocBug(_bugNr, _ptr, _size) \
-   Util_SafeInternalRealloc((_bugNr), (_ptr), (_size), __FILE__, __LINE__)
+   UtilSafeRealloc1((_ptr), (_size), (_bugNr), __FILE__, __LINE__)
 
 #define Util_SafeCalloc(_nmemb, _size) \
-   Util_SafeInternalCalloc(-1, (_nmemb), (_size), __FILE__, __LINE__)
+   UtilSafeCalloc1((_nmemb), (_size), -1, __FILE__, __LINE__)
 
 #define Util_SafeCallocBug(_bugNr, _nmemb, _size) \
-   Util_SafeInternalCalloc((_bugNr), (_nmemb), (_size), __FILE__, __LINE__)
+   UtilSafeCalloc1((_nmemb), (_size), (_bugNr), __FILE__, __LINE__)
 
 #define Util_SafeStrndup(_str, _size) \
-   Util_SafeInternalStrndup(-1, (_str), (_size), __FILE__, __LINE__)
+   UtilSafeStrndup1((_str), (_size), -1, __FILE__, __LINE__)
 
 #define Util_SafeStrndupBug(_bugNr, _str, _size) \
-   Util_SafeInternalStrndup((_bugNr), (_str), (_size), __FILE__, __LINE__)
+   UtilSafeStrndup1((_str), (_size), (_bugNr), __FILE__, __LINE__)
 
 #define Util_SafeStrdup(_str) \
-   Util_SafeInternalStrdup(-1, (_str), __FILE__, __LINE__)
+   UtilSafeStrdup1((_str), -1, __FILE__, __LINE__)
 
 #define Util_SafeStrdupBug(_bugNr, _str) \
-   Util_SafeInternalStrdup((_bugNr), (_str), __FILE__, __LINE__)
+   UtilSafeStrdup1((_str), (_bugNr), __FILE__, __LINE__)
+
+#else  /* VMX86_DEBUG */
+
+#define Util_SafeMalloc(_size) \
+   UtilSafeMalloc0((_size))
+
+#define Util_SafeMallocBug(_bugNr, _size) \
+   UtilSafeMalloc0((_size))
+
+#define Util_SafeRealloc(_ptr, _size) \
+   UtilSafeRealloc0((_ptr), (_size))
+
+#define Util_SafeReallocBug(_ptr, _size) \
+   UtilSafeRealloc0((_ptr), (_size))
+
+#define Util_SafeCalloc(_nmemb, _size) \
+   UtilSafeCalloc0((_nmemb), (_size))
+
+#define Util_SafeCallocBug(_bugNr, _nmemb, _size) \
+   UtilSafeCalloc0((_nmemb), (_size))
+
+#define Util_SafeStrndup(_str, _size) \
+   UtilSafeStrndup0((_str), (_size))
+
+#define Util_SafeStrndupBug(_bugNr, _str, _size) \
+   UtilSafeStrndup0((_str), (_size))
+
+#define Util_SafeStrdup(_str) \
+   UtilSafeStrdup0((_str))
+
+#define Util_SafeStrdupBug(_bugNr, _str) \
+   UtilSafeStrdup0((_str))
+
+#endif  /* VMX86_DEBUG */
+
+
+void *Util_Memcpy(void *dest, const void *src, size_t count);
 
 
 /*
@@ -547,4 +605,30 @@ Util_FreeStringList(char **list,      // IN/OUT: the list to free
 {
    Util_FreeList((void **) list, length);
 }
+
+#ifndef _WIN32
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * Util_IsFileDescriptorOpen --
+ *
+ *      Check if given file descriptor is open.
+ *
+ * Results:
+ *      TRUE if fd is open.
+ *
+ * Side effects:
+ *      Clobbers errno.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+static INLINE Bool
+Util_IsFileDescriptorOpen(int fd)   // IN
+{
+   lseek(fd, 0L, SEEK_CUR);
+   return errno != EBADF;
+}
+#endif /* !_WIN32 */
+
 #endif /* UTIL_H */
