@@ -33,25 +33,35 @@
 #define UUID_STRSIZE (2*UUID_SIZE + 1)
 #define	UUID_MAXLEN 48
 
+typedef enum {
+   UUID_WITH_PATH = 0,
+   UUID_RANDOM,
+   UUID_VPX_BIOS,
+   UUID_VPX_INSTANCE,
+   UUID_UNKNOWN
+} UUIDStyle;
+
 Bool UUID_ConvertToBin(uint8 dest_id[UUID_SIZE], const char *text);
 char *UUID_ConvertToText(const uint8 id[UUID_SIZE]);
 
 #define UUID_CREATE_WS4     0  /* the "original", WS4 and earlier scheme */
 #define UUID_CREATE_WS5     1  /* the WS5 scheme */
 #define UUID_CREATE_WS6     2  /* the WS6 scheme - "native" path */
-#define UUID_CREATE_WS65    3  /* the WS65 scheme - UTF-8 path */
-#define UUID_CREATE_CURRENT 3  /* the current scheme - always the latest */
+#define UUID_CREATE_ESX50   3  /* the scheme to allow location generated using
+                                  host UUID with wrong endianness as reported by
+                                  pre-ESX 5.0 U2. See PR 861271 for details. */
+#define UUID_CREATE_WS65    4  /* the WS65 scheme - UTF-8 path */
+#define UUID_CREATE_CURRENT 4  /* the current scheme - always the latest */
 
 char *UUID_Create(const char *configFileFullPath, int schemeControl);
 
 char *UUID_CreateRandom(void);
-char *UUID_CreateRandomVpxStyle(uint8 vpxdId);
+char *UUID_CreateRandomVpxStyle(uint8 vpxdId, UUIDStyle);
 Bool UUID_IsUUIDGeneratedByThatVpxd(const uint8 *id, int vpxdInstanceId);
 char *UUID_PackText(const char *text, char *pack, int packLen);
 char *UUID_ProperHostUUID(void);
 char *UUID_GetHostUUID(void);
-#if defined(_WIN32)
+UUIDStyle UUID_GetStyle(const uint8 *id);
 /* like UUID_GetHostUUID, except gets actual host UUID */
-char *UUID_GetRealHostUUID(void); 
-#endif /* _WIN32 */ 
+char *UUID_GetRealHostUUID(void);
 #endif
