@@ -232,7 +232,7 @@ MXUserWaitInternal(MXRecLock *lock,         // IN:
                    MXUserCondVar *condVar,  // IN:
                    uint32 msecWait)         // IN:
 {
-   uint32 lockCount = MXRecLockCount(lock);
+   int lockCount = MXRecLockCount(lock);
    DWORD waitTime = (msecWait == MXUSER_WAIT_INFINITE) ? INFINITE : msecWait;
 
    if (pSleepConditionVariableCS) {
@@ -300,7 +300,9 @@ MXUserWaitInternal(MXRecLock *lock,         // IN:
          LeaveCriticalSection(&condVar->x.compat.condVarLock);
       } while (!done);
 
-      MXRecLockAcquire(lock);
+      MXRecLockAcquire(lock,
+                       NULL);  // non-stats
+
       MXRecLockIncCount(lock, lockCount - 1);
 
       if (err != ERROR_SUCCESS) {
@@ -461,7 +463,7 @@ MXUserWaitInternal(MXRecLock *lock,         // IN:
                    uint32 msecWait)         // IN:
 {
    int err;
-   uint32 lockCount = MXRecLockCount(lock);
+   int lockCount = MXRecLockCount(lock);
 
    /*
     * When using the native lock found within the MXUser lock, be sure to
@@ -632,7 +634,7 @@ MXUserWaitCondVar(MXUserHeader *header,    // IN:
              __FUNCTION__, header->name, condVar, condVar->header->name);
    }
 
-   if (!MXRecLockIsOwner(lock)) {
+   if (vmx86_debug && !MXRecLockIsOwner(lock)) {
       Panic("%s: lock %s for condVar (0x%p) not owned\n",
             __FUNCTION__, condVar->header->name, condVar);
    }
