@@ -29,7 +29,6 @@
 #define _XOPEN_SOURCE 600
 #endif
 #include <semaphore.h>
-#include <time.h>
 #include <sys/time.h>
 #endif
 #endif
@@ -44,7 +43,6 @@
 #include "win32u.h"
 #endif
 
-#define MXUSER_SEMA_SIGNATURE 0x414D4553 // 'SEMA' in memory
 #define MXUSER_A_BILLION (1000 * 1000 * 1000)
 
 #if defined(_WIN32)
@@ -59,16 +57,16 @@ typedef sem_t NativeSemaphore;
 
 typedef struct
 {
-   MXUserAcquisitionStats  acquisitionStats;
-   Atomic_Ptr              acquisitionHisto;
-} MXUserStats;
+   MXUserAcquisitionStats  data;
+   Atomic_Ptr              histo;
+} MXUserAcquireStats;
 
 struct MXUserSemaphore
 {
    MXUserHeader     header;
    Atomic_uint32    activeUserCount;
    NativeSemaphore  nativeSemaphore;
-   Atomic_Ptr       statsMem;
+   Atomic_Ptr       acquireStatsMem;
 };
 
 
@@ -126,7 +124,7 @@ MXUserTimedDown(NativeSemaphore *sema,  // IN:
 
     status = WaitForSingleObject(*sema, msecWait);
 
-    switch (status) { 
+    switch (status) {
        case WAIT_OBJECT_0:  // The down (decrement) occurred
           *downOccurred = TRUE;
           err = 0;
@@ -405,9 +403,9 @@ static void
 MXUserStatsActionSema(MXUserHeader *header)  // IN:
 {
    MXUserSemaphore *sema = (MXUserSemaphore *) header;
-   MXUserStats *stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+   MXUserAcquireStats *acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
 
-   if (stats) {
+   if (LIKELY(acquireStats != NULL)) {
       Bool isHot;
       Bool doLog;
       double contentionRatio;
@@ -416,21 +414,20 @@ MXUserStatsActionSema(MXUserHeader *header)  // IN:
        * Dump the statistics for the specified semaphore.
        */
 
-      MXUserDumpAcquisitionStats(&stats->acquisitionStats, header);
+      MXUserDumpAcquisitionStats(&acquireStats->data, header);
 
-      if (Atomic_ReadPtr(&stats->acquisitionHisto) != NULL) {
-         MXUserHistoDump(Atomic_ReadPtr(&stats->acquisitionHisto), header);
+      if (Atomic_ReadPtr(&acquireStats->histo) != NULL) {
+         MXUserHistoDump(Atomic_ReadPtr(&acquireStats->histo), header);
       }
 
       /*
        * Has the semaphore gone "hot"? If so, implement the hot actions.
        */
 
-      MXUserKitchen(&stats->acquisitionStats, &contentionRatio, &isHot,
-                    &doLog);
+      MXUserKitchen(&acquireStats->data, &contentionRatio, &isHot, &doLog);
 
       if (isHot) {
-         MXUserForceHisto(&stats->acquisitionHisto,
+         MXUserForceHisto(&acquireStats->histo,
                           MXUSER_STAT_CLASS_ACQUISITION,
                           MXUSER_DEFAULT_HISTO_MIN_VALUE_NS,
                           MXUSER_DEFAULT_HISTO_DECADES);
@@ -465,7 +462,7 @@ MXUserDumpSemaphore(MXUserHeader *header)  // IN:
 {
    MXUserSemaphore *sema = (MXUserSemaphore *) header;
 
-   Warning("%s: semaphore @ 0x%p\n", __FUNCTION__, sema);
+   Warning("%s: semaphore @ %p\n", __FUNCTION__, sema);
 
    Warning("\tsignature 0x%X\n", sema->header.signature);
    Warning("\tname %s\n", sema->header.name);
@@ -473,7 +470,7 @@ MXUserDumpSemaphore(MXUserHeader *header)  // IN:
    Warning("\tserial number %u\n", sema->header.serialNumber);
 
    Warning("\treference count %u\n", Atomic_Read(&sema->activeUserCount));
-   Warning("\tnative semaphore 0x%p\n", &sema->nativeSemaphore);
+   Warning("\taddress of native semaphore %p\n", &sema->nativeSemaphore);
 }
 
 
@@ -513,25 +510,24 @@ MXUser_CreateSemaphore(const char *userName,  // IN:
    }
 
    if (LIKELY(MXUserInit(&sema->nativeSemaphore) == 0)) {
-      MXUserStats *stats;
-
-      sema->header.signature = MXUSER_SEMA_SIGNATURE;
+      sema->header.signature = MXUserGetSignature(MXUSER_TYPE_SEMA);
       sema->header.name = properName;
       sema->header.rank = rank;
       sema->header.serialNumber = MXUserAllocSerialNumber();
       sema->header.dumpFunc = MXUserDumpSemaphore;
 
-      if (MXUserStatsEnabled()) {
-         sema->header.statsFunc = MXUserStatsActionSema;
-
-         stats = Util_SafeCalloc(1, sizeof(*stats));
-
-         MXUserAcquisitionStatsSetUp(&stats->acquisitionStats);
-
-         Atomic_WritePtr(&sema->statsMem, stats);
-      } else {
+      if (MXUserStatsMode() == 0) {
          sema->header.statsFunc = NULL;
-         Atomic_WritePtr(&sema->statsMem, NULL);
+         Atomic_WritePtr(&sema->acquireStatsMem, NULL);
+      } else {
+         MXUserAcquireStats *acquireStats;
+
+         acquireStats = Util_SafeCalloc(1, sizeof(*acquireStats));
+
+         MXUserAcquisitionStatsSetUp(&acquireStats->data);
+
+         sema->header.statsFunc = MXUserStatsActionSema;
+         Atomic_WritePtr(&sema->acquireStatsMem, acquireStats);
       }
 
       MXUserAddToList(&sema->header);
@@ -566,9 +562,8 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
 {
    if (LIKELY(sema != NULL)) {
       int err;
-      MXUserStats *stats;
 
-      ASSERT(sema->header.signature == MXUSER_SEMA_SIGNATURE);
+      MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
 
       if (Atomic_Read(&sema->activeUserCount) != 0) {
          MXUserDumpAndPanic(&sema->header,
@@ -576,6 +571,8 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
                             __FUNCTION__);
       }
 
+      sema->header.signature = 0;  // just in case...
+
       err = MXUserDestroy(&sema->nativeSemaphore);
 
       if (UNLIKELY(err != 0)) {
@@ -585,16 +582,19 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
 
       MXUserRemoveFromList(&sema->header);
 
-      stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+      if (vmx86_stats) {
+         MXUserAcquireStats *acquireStats;
 
-      if (stats) {
-         MXUserAcquisitionStatsTearDown(&stats->acquisitionStats);
-         MXUserHistoTearDown(Atomic_ReadPtr(&stats->acquisitionHisto));
+         acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
 
-         free(stats);
+         if (LIKELY(acquireStats != NULL)) {
+            MXUserAcquisitionStatsTearDown(&acquireStats->data);
+            MXUserHistoTearDown(Atomic_ReadPtr(&acquireStats->histo));
+
+            free(acquireStats);
+         }
       }
 
-      sema->header.signature = 0;  // just in case...
       free(sema->header.name);
       sema->header.name = NULL;
       free(sema);
@@ -607,7 +607,7 @@ MXUser_DestroySemaphore(MXUserSemaphore *sema)  // IN:
  *
  * MXUser_DownSemaphore --
  *
- *      Perform an down (P; probeer te verlagen; "try to reduce") operation
+ *      Perform a down (P; probeer te verlagen; "try to reduce") operation
  *      on a semaphore.
  *
  * Results:
@@ -624,19 +624,24 @@ void
 MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
-   MXUserStats *stats;
 
-   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
+   ASSERT(sema);
+   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
 
    Atomic_Inc(&sema->activeUserCount);
 
    MXUserAcquisitionTracking(&sema->header, TRUE);  // rank checking
 
-   stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
-
-   if (stats) { 
+   if (vmx86_stats) {
+      VmTimeType start = 0;
       Bool tryDownSuccess = FALSE;
-      VmTimeType begin = Hostinfo_SystemTimerNS();
+      MXUserAcquireStats *acquireStats;
+
+      acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
+
+      if (LIKELY(acquireStats != NULL)) {
+         start = Hostinfo_SystemTimerNS();
+      }
 
       err = MXUserTryDown(&sema->nativeSemaphore, &tryDownSuccess);
 
@@ -644,19 +649,19 @@ MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
          if (!tryDownSuccess) {
             err = MXUserDown(&sema->nativeSemaphore);
          }
+      }
 
-         if (LIKELY(err == 0)) {
-            MXUserHisto *histo;
-            VmTimeType value = Hostinfo_SystemTimerNS() - begin;
+      if (LIKELY((err == 0) && (acquireStats != NULL))) {
+         MXUserHisto *histo;
+         VmTimeType value = Hostinfo_SystemTimerNS() - start;
 
-            MXUserAcquisitionSample(&stats->acquisitionStats, TRUE,
-                                    !tryDownSuccess, value);
+         MXUserAcquisitionSample(&acquireStats->data, TRUE,
+                                 !tryDownSuccess, value);
 
-            histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+         histo = Atomic_ReadPtr(&acquireStats->histo);
 
-            if (UNLIKELY(histo != NULL)) {
-               MXUserHistoSample(histo, value, GetReturnAddress());
-            }
+         if (UNLIKELY(histo != NULL)) {
+            MXUserHistoSample(histo, value, GetReturnAddress());
          }
       }
    } else {
@@ -679,7 +684,7 @@ MXUser_DownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
  *
  * MXUser_TimedDownSemaphore --
  *
- *      Perform an down (P; probeer te verlagen; "try to reduce") operation
+ *      Perform a down (P; probeer te verlagen; "try to reduce") operation
  *      on a semaphore with a timeout. The wait time will always have elapsed
  *      before the routine returns.
  *
@@ -698,20 +703,25 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
                           uint32 msecWait)        // IN:
 {
    int err;
-   MXUserStats *stats;
    Bool downOccurred = FALSE;
 
-   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
+   ASSERT(sema);
+   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
 
    Atomic_Inc(&sema->activeUserCount);
 
    MXUserAcquisitionTracking(&sema->header, TRUE);  // rank checking
 
-   stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
-
-   if (stats) { 
+   if (vmx86_stats) {
+      VmTimeType start = 0;
       Bool tryDownSuccess = FALSE;
-      VmTimeType begin = Hostinfo_SystemTimerNS();
+      MXUserAcquireStats *acquireStats;
+
+      acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
+
+      if (LIKELY(acquireStats != NULL)) {
+         start = Hostinfo_SystemTimerNS();
+      }
 
       err = MXUserTryDown(&sema->nativeSemaphore, &tryDownSuccess);
 
@@ -722,19 +732,19 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
             err = MXUserTimedDown(&sema->nativeSemaphore, msecWait,
                                   &downOccurred);
          }
+      }
 
-         if (LIKELY(err == 0)) {
-            VmTimeType value = Hostinfo_SystemTimerNS() - begin;
+      if (LIKELY((err == 0) && (acquireStats != NULL))) {
+         VmTimeType value = Hostinfo_SystemTimerNS() - start;
 
-            MXUserAcquisitionSample(&stats->acquisitionStats, downOccurred,
-                                    !tryDownSuccess, value);
+         MXUserAcquisitionSample(&acquireStats->data, downOccurred,
+                                 !tryDownSuccess, value);
 
-            if (downOccurred) {
-               MXUserHisto *histo = Atomic_ReadPtr(&stats->acquisitionHisto);
+         if (downOccurred) {
+            MXUserHisto *histo = Atomic_ReadPtr(&acquireStats->histo);
 
-               if (UNLIKELY(histo != NULL)) {
-                  MXUserHistoSample(histo, value, GetReturnAddress());
-               }
+            if (UNLIKELY(histo != NULL)) {
+               MXUserHistoSample(histo, value, GetReturnAddress());
             }
          }
       }
@@ -760,7 +770,7 @@ MXUser_TimedDownSemaphore(MXUserSemaphore *sema,  // IN/OUT:
  *
  * MXUser_TryDownSemaphore --
  *
- *      Perform an try down (P; probeer te verlagen; "try to reduce") operation
+ *      Perform a try down (P; probeer te verlagen; "try to reduce") operation
  *      on a semaphore.
  *
  * Results:
@@ -781,10 +791,10 @@ Bool
 MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
-   MXUserStats *stats;
    Bool downOccurred = FALSE;
 
-   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
+   ASSERT(sema);
+   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
 
    Atomic_Inc(&sema->activeUserCount);
 
@@ -795,11 +805,15 @@ MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
                          __FUNCTION__, err);
    }
 
-   stats = (MXUserStats *) Atomic_ReadPtr(&sema->statsMem);
+   if (vmx86_stats) {
+      MXUserAcquireStats *acquireStats;
 
-   if (stats) {
-      MXUserAcquisitionSample(&stats->acquisitionStats, downOccurred,
-                              !downOccurred, 0ULL);
+      acquireStats = Atomic_ReadPtr(&sema->acquireStatsMem);
+
+      if (LIKELY(acquireStats != NULL)) {
+         MXUserAcquisitionSample(&acquireStats->data, downOccurred,
+                                 !downOccurred, 0ULL);
+      }
    }
 
    Atomic_Dec(&sema->activeUserCount);
@@ -814,7 +828,7 @@ MXUser_TryDownSemaphore(MXUserSemaphore *sema)  // IN/OUT:
  * MXUser_UpSemaphore --
  *
  *      Perform an up (V; verhogen; "increase") operation on a semaphore.
- * 
+ *
  * Results:
  *      The semaphore count is incremented. Any thread waiting on the
  *      semaphore is awoken.
@@ -830,7 +844,8 @@ MXUser_UpSemaphore(MXUserSemaphore *sema)  // IN/OUT:
 {
    int err;
 
-   ASSERT(sema && (sema->header.signature == MXUSER_SEMA_SIGNATURE));
+   ASSERT(sema);
+   MXUserValidateHeader(&sema->header, MXUSER_TYPE_SEMA);
 
    Atomic_Inc(&sema->activeUserCount);
 
@@ -874,19 +889,17 @@ MXUser_CreateSingletonSemaphore(Atomic_Ptr *semaStorage,  // IN/OUT:
 
    ASSERT(semaStorage);
 
-   sema = (MXUserSemaphore *) Atomic_ReadPtr(semaStorage);
+   sema = Atomic_ReadPtr(semaStorage);
 
    if (UNLIKELY(sema == NULL)) {
       MXUserSemaphore *newSema = MXUser_CreateSemaphore(name, rank);
 
-      sema = (MXUserSemaphore *) Atomic_ReadIfEqualWritePtr(semaStorage,
-                                                            NULL,
-                                                            (void *) newSema);
+      sema = Atomic_ReadIfEqualWritePtr(semaStorage, NULL, (void *) newSema);
 
       if (sema) {
          MXUser_DestroySemaphore(newSema);
       } else {
-         sema = (MXUserSemaphore *) Atomic_ReadPtr(semaStorage);
+         sema = Atomic_ReadPtr(semaStorage);
       }
    }
 
