@@ -74,9 +74,11 @@
 #include "vmware.h"
 #include "vm_product.h"
 #include "vm_atomic.h"
-#include "unicode/ucnv.h"
-#include "unicode/udata.h"
-#include "unicode/putil.h"
+#if !defined(NO_ICU)
+#  include "unicode/ucnv.h"
+#  include "unicode/udata.h"
+#  include "unicode/putil.h"
+#endif
 #include "file.h"
 #include "util.h"
 #include "codeset.h"
@@ -137,6 +139,8 @@ static Bool dontUseIcu = TRUE;
  * Functions
  */
 
+#if !defined NO_ICU
+
 #ifdef _WIN32
 
 /*
@@ -166,7 +170,7 @@ CodeSetGetModulePath(HANDLE hModule) // IN
    while (TRUE) {
       DWORD res;
 
-      pathW = realloc(pathW, size * sizeof(wchar_t));
+      pathW = realloc(pathW, size * (sizeof *pathW));
       if (!pathW) {
          return NULL;
       }
@@ -177,7 +181,13 @@ CodeSetGetModulePath(HANDLE hModule) // IN
          /* fatal error */
          goto exit;
       } else if (res == size) {
-         /* buffer too small */
+         /*
+          * The buffer is too small; double its size. The maximum path length
+          * on Windows is ~64KB so doubling will not push a DWORD into
+          * integer overflow before success or error due to another reason
+          * will occur.
+          */
+
          size *= 2;
       } else {
          /* success */
@@ -202,7 +212,7 @@ CodeSetGetModulePath(HANDLE hModule) // IN
  *	use before ICU is initialized.
  *
  * Results:
- *      On success: The allocated, NUL-terminated file path.  
+ *      On success: The allocated, NUL-terminated file path.
  *         Note: This path can be a symbolic or hard link; it's just one
  *         possible path to access the executable.
  *         
@@ -247,7 +257,7 @@ CodeSetGetModulePath(uint32 priv)
       uid = Id_BeginSuperUser();
    }
 
-   size = readlink("/proc/self/exe", path, sizeof path);
+   size = readlink("/proc/self/exe", path, sizeof path - 1);
    if (-1 == size) {
       if (priv == HGMP_PRIVILEGE) {
          Id_EndSuperUser(uid);
@@ -274,6 +284,8 @@ CodeSetGetModulePath(uint32 priv)
 
 #endif // _WIN32
 
+#endif // !NO_ICU
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -301,11 +313,14 @@ CodeSet_GetAltPathName(const utf16_t *pathW) // IN
 {
    char *path = NULL;
 #if defined(_WIN32)
+   DWORD res;
    utf16_t shortPathW[_MAX_PATH];
 
    ASSERT(pathW);
 
-   if (GetShortPathNameW(pathW, shortPathW, ARRAYSIZE(shortPathW)) == 0) {
+   res = GetShortPathNameW(pathW, shortPathW, ARRAYSIZE(shortPathW));
+
+   if ((res == 0) || (res >= ARRAYSIZE(shortPathW))) {
       goto exit;
    }
 
@@ -383,6 +398,10 @@ Bool
 CodeSet_Init(const char *icuDataDir) // IN: ICU data file location in Current code page.
                                      //     Default is used if NULL.
 {
+#ifdef NO_ICU
+   /* Nothing required if not using ICU. */
+   return TRUE;
+#else // NO_ICU
    DynBuf dbpath;
 #ifdef _WIN32
    DWORD attribs;
@@ -690,6 +709,7 @@ found:
    DynBuf_Destroy(&dbpath);
 
    return ret;
+#endif
 }
 
 
@@ -716,17 +736,22 @@ static Bool
 CodeSetDuplicateUtf8Str(const char *bufIn,  // IN: Input string
                         size_t sizeIn,      // IN: Input string length
                         char **bufOut,      // OUT: "Converted" string
-                        size_t *sizeOut)    // OUT: Length of string
+                        size_t *sizeOut)    // OUT/OPT: Length of string
 {
    char *myBufOut;
+   size_t newSize = sizeIn + sizeof *myBufOut;
 
-   myBufOut = malloc(sizeIn + 1);
+   if (newSize < sizeIn) {   // Prevent integer overflow
+      return FALSE;
+   }
+
+   myBufOut = malloc(newSize);
    if (myBufOut == NULL) {
       return FALSE;
    }
 
    memcpy(myBufOut, bufIn, sizeIn);
-   memset(myBufOut + sizeIn, 0, 1);
+   myBufOut[sizeIn] = '\0';
 
    *bufOut = myBufOut;
    if (sizeOut) {
@@ -761,7 +786,7 @@ static Bool
 CodeSetDynBufFinalize(Bool ok,          // IN: Earlier steps succeeded
                       DynBuf *db,       // IN: Buffer with converted string
                       char **bufOut,    // OUT: Converted string
-                      size_t *sizeOut)  // OUT: Length of string in bytes
+                      size_t *sizeOut)  // OUT/OPT: Length of string in bytes
 {
    /*
     * NUL can be as long as 4 bytes if UTF-32, make no assumptions.
@@ -837,30 +862,6 @@ CodeSet_Utf8Normalize(const char *bufIn,     // IN
 #endif /* defined(__APPLE__) */
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * CodeSet_GetCurrentCodeSet --
- *
- *    Return native code set name. Always calls down to
- *    CodeSetOld_GetCurrentCodeSet. See there for more details.
- *
- * Results:
- *    See CodeSetOld_GetCurrentCodeSet.
- *
- * Side effects:
- *    See CodeSetOld_GetCurrentCodeSet.
- *
- *-----------------------------------------------------------------------------
- */
-
-const char *
-CodeSet_GetCurrentCodeSet(void)
-{
-   return CodeSetOld_GetCurrentCodeSet();
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -887,6 +888,10 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
                            unsigned int flags,  // IN
                            DynBuf *db)          // IN/OUT
 {
+#if defined(NO_ICU)
+   return CodeSetOld_GenericToGenericDb(codeIn, bufIn, sizeIn, codeOut,
+                                        flags, db);
+#else
    Bool result = FALSE;
    UErrorCode uerr;
    const char *bufInCur;
@@ -898,6 +903,7 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
    char *bufOut;
    char *bufOutCur;
    char *bufOutEnd;
+   size_t newSize;
    size_t bufOutSize;
    size_t bufOutOffset;
    UConverter *cvin = NULL;
@@ -989,7 +995,11 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
 
    bufInCur = bufIn;
    bufInEnd = bufIn + sizeIn;
-   bufOutSize = sizeIn + 4;
+   newSize = sizeIn + 4;
+   if (newSize < sizeIn) {  // Prevent integer overflow
+      goto exit;
+   }
+   bufOutSize = newSize;
    bufOutOffset = 0;
    bufPivSource = bufPiv;
    bufPivTarget = bufPiv;
@@ -1012,11 +1022,8 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
 
       if (!U_FAILURE(uerr)) {
          /*
-          * "This was a triumph.
-          *  I'm making a note here:
-          *  HUGE SUCCESS.
-          *  It's hard to overstate
-          *  my satisfaction."
+          * "This was a triumph. I'm making a note here: HUGE SUCCESS. It's
+          * hard to overstate my satisfaction."
           */
 
          break;
@@ -1033,7 +1040,19 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
        * based on bufOutOffset.
        */
 
-      bufOutSize *= 2;
+      newSize = 2 * bufOutSize;
+
+      /*
+       * Prevent integer overflow. We can use this form of checking
+       * specifically because a multiple by 2 is used. This type of checking
+       * does not work in the general case.
+       */
+
+      if (newSize < bufOutSize) {
+         goto exit;
+      }
+
+      bufOutSize = newSize;
       bufOutOffset = bufOutCur - bufOut;
    }
 
@@ -1055,6 +1074,7 @@ CodeSet_GenericToGenericDb(const char *codeIn,  // IN
    }
 
    return result;
+#endif
 }
 
 
@@ -1604,6 +1624,9 @@ CodeSet_Utf16beToCurrent(const char *bufIn,  // IN
 Bool
 CodeSet_IsEncodingSupported(const char *name) // IN
 {
+#if defined(NO_ICU)
+   return CodeSetOld_IsEncodingSupported(name);
+#else
    UConverter *cv;
    UErrorCode uerr;
 
@@ -1626,6 +1649,7 @@ CodeSet_IsEncodingSupported(const char *name) // IN
    }
 
    return FALSE;
+#endif
 }
 
 
@@ -1651,6 +1675,9 @@ CodeSet_Validate(const char *buf,   // IN: the string
                  size_t size,	    // IN: length of string
                  const char *code)  // IN: encoding
 {
+#if defined(NO_ICU)
+   return CodeSetOld_Validate(buf, size, code);
+#else
    UConverter *cv;
    UErrorCode uerr;
 
@@ -1685,5 +1712,6 @@ CodeSet_Validate(const char *buf,   // IN: the string
    ucnv_close(cv);
 
    return uerr == U_BUFFER_OVERFLOW_ERROR;
+#endif
 }
 
