@@ -169,6 +169,7 @@ typedef struct distro_info {
 } DistroInfo;
 
 static const DistroInfo distroArray[] = {
+   {"OracleLinux",        "/etc/oracle-release"},
    {"RedHat",             "/etc/redhat-release"},
    {"RedHat",             "/etc/redhat_version"},
    {"Sun",                "/etc/sun-release"},
@@ -523,8 +524,11 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
       Str_Strcpy(distroShort, STR_OS_OPENSUSE, distroShortSize);
    } else if (strstr(distroLower, "suse")) {
       if (strstr(distroLower, "enterprise")) {
-         if (strstr(distroLower, "server 11") ||
-             strstr(distroLower, "desktop 11")) {
+         if (strstr(distroLower, "server 12") ||
+             strstr(distroLower, "desktop 12")) {
+            Str_Strcpy(distroShort, STR_OS_SLES_12, distroShortSize);
+         } else if (strstr(distroLower, "server 11") ||
+                    strstr(distroLower, "desktop 11")) {
             Str_Strcpy(distroShort, STR_OS_SLES_11, distroShortSize);
          } else if (strstr(distroLower, "server 10") ||
                     strstr(distroLower, "desktop 10")) {
@@ -574,6 +578,8 @@ HostinfoGetOSShortName(char *distro,         // IN: full distro name
          Str_Strcpy(distroShort, STR_OS_DEBIAN_5, distroShortSize);
       } else if (strstr(distroLower, "6.0")) {
          Str_Strcpy(distroShort, STR_OS_DEBIAN_6, distroShortSize);
+      } else if (strstr(distroLower, "7.0")) {
+         Str_Strcpy(distroShort, STR_OS_DEBIAN_7, distroShortSize);
       }
    } else if (StrUtil_StartsWith(distroLower, "enterprise linux") ||
               StrUtil_StartsWith(distroLower, "oracle")) {
@@ -911,9 +917,12 @@ HostinfoOSData(void)
       } else if (majorVersion == 2 && Hostinfo_OSVersion(1) < 6) {
          Str_Strcpy(distro, STR_OS_OTHER_24_FULL, distroSize);
          Str_Strcpy(distroShort, STR_OS_OTHER_24, distroSize);
-      } else {
+      } else if (majorVersion == 2) {
          Str_Strcpy(distro, STR_OS_OTHER_26_FULL, distroSize);
          Str_Strcpy(distroShort, STR_OS_OTHER_26, distroSize);
+      } else {
+         Str_Strcpy(distro, STR_OS_OTHER_3X_FULL, distroSize);
+         Str_Strcpy(distroShort, STR_OS_OTHER_3X, distroSize);
       }
 
       /*
