@@ -21,6 +21,11 @@
 #endif
 
 #define UNICODE_BUILDING_POSIX_WRAPPERS
+
+#if __linux__
+#define _GNU_SOURCE // Needed to get euidaccess()
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
@@ -34,10 +39,8 @@
 #include <utime.h>
 #include <sys/time.h>
 #include <stdarg.h>
-#if !defined(N_PLAT_NLM)
 #include <pwd.h>
 #include <grp.h>
-#endif
 
 #if defined(__APPLE__)
 #include <sys/socket.h>
@@ -53,18 +56,16 @@
 #elif defined(__FreeBSD__)
 #include <sys/param.h>
 #include <sys/mount.h>
-#elif !defined(N_PLAT_NLM)
-#include <sys/statfs.h>
-#include <sys/mount.h>
-#if !defined(sun)
-#include <mntent.h>
-#else
+#elif defined(sun)
 #include <alloca.h>
 #include <sys/mnttab.h>
-#endif
+#else
+#include <sys/statfs.h>
+#include <sys/mount.h>
+#include <mntent.h>
 #endif
 
-#if !defined(__FreeBSD__) || __FreeBSD_release >= 503001
+#if (!defined(__FreeBSD__) || __FreeBSD_release >= 503001) && !defined __ANDROID__
 #define VM_SYSTEM_HAS_GETPWNAM_R 1
 #define VM_SYSTEM_HAS_GETPWUID_R 1
 #define VM_SYSTEM_HAS_GETGRNAM_R 1
@@ -82,10 +83,21 @@
 #include "hashTable.h" // For setenv emulation
 #endif
 
-#if !defined(N_PLAT_NLM)
+#include "vm_basic_defs.h"
+
 static struct passwd *GetpwInternal(struct passwd *pw);
 static int GetpwInternal_r(struct passwd *pw, char *buf, size_t size,
                            struct passwd **ppw);
+
+#if defined __ANDROID__
+/*
+ * Android doesn't support getmntent_r(), getpwent() or setmntent().
+ */
+#define NO_GETMNTENT_R
+#define NO_GETPWENT
+#define NO_SETMNTENT
+
+EXTERN int truncate(const char *, off_t);
 #endif
 
 
@@ -138,6 +150,7 @@ Posix_Open(ConstUnicode pathName,  // IN:
    fd = open(path, flags, mode);
 
    free(path);
+
    return fd;
 }
 
@@ -160,8 +173,8 @@ Posix_Open(ConstUnicode pathName,  // IN:
  */
 
 int
-Posix_Creat(ConstUnicode pathName, // IN:
-            mode_t mode)           // IN:
+Posix_Creat(ConstUnicode pathName,  // IN:
+            mode_t mode)            // IN:
 {
    return Posix_Open(pathName, O_CREAT | O_WRONLY | O_TRUNC, mode);
 }
@@ -184,8 +197,8 @@ Posix_Creat(ConstUnicode pathName, // IN:
  */
 
 FILE *
-Posix_Fopen(ConstUnicode pathName, // IN:
-            const char *mode)      // IN:
+Posix_Fopen(ConstUnicode pathName,  // IN:
+            const char *mode)       // IN:
 {
    char *path;
    FILE *stream;
@@ -199,6 +212,7 @@ Posix_Fopen(ConstUnicode pathName, // IN:
    stream = fopen(path, mode);
 
    free(path);
+
    return stream;
 }
 
@@ -234,6 +248,7 @@ Posix_Stat(ConstUnicode pathName,  // IN:
    ret = stat(path, statbuf);
 
    free(path);
+
    return ret;
 }
 
@@ -378,6 +393,7 @@ Posix_Rmdir(ConstUnicode pathName)  // IN:
    ret = rmdir(path);
 
    free(path);
+
    return ret;
 }
 
@@ -400,9 +416,9 @@ Posix_Rmdir(ConstUnicode pathName)  // IN:
  */
 
 FILE *
-Posix_Freopen(ConstUnicode pathName, // IN:
-              const char *mode,      // IN:
-              FILE *input_stream)    // IN:
+Posix_Freopen(ConstUnicode pathName,  // IN:
+              const char *mode,       // IN:
+              FILE *input_stream)     // IN:
 {
    char *path;
    FILE *stream;
@@ -448,10 +464,63 @@ Posix_Access(ConstUnicode pathName,  // IN:
       return -1;
    }
 
+#if defined(VMX86_SERVER)
+   /*
+    * ESX can return EINTR making retries a necessity. This is a bug in
+    * ESX that is being worked around here - POSIX says access cannot return
+    * EINTR.
+    */
+
+   do {
+      ret = access(path, mode);
+   } while ((ret == -1) && (errno == EINTR));
+#else
    ret = access(path, mode);
+#endif
+
+   free(path);
+
+   return ret;
+}
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Posix_EuidAccess --
+ *
+ *      POSIX euidaccess().
+ *
+ * Results:
+ *      -1	Error
+ *      0	Success
+ *
+ * Side effects:
+ *      errno is set on error
+ *
+ *----------------------------------------------------------------------
+ */
+
+int
+Posix_EuidAccess(ConstUnicode pathName,  // IN:
+                 int mode)               // IN:
+{
+#if defined(GLIBC_VERSION_24)
+   char *path;
+   int ret;
+
+   if (!PosixConvertToCurrent(pathName, &path)) {
+      return -1;
+   }
+
+   ret = euidaccess(path, mode);
 
    free(path);
    return ret;
+#else
+   errno = ENOSYS;
+   return -1;
+#endif
 }
 
 
@@ -486,6 +555,7 @@ Posix_Utime(ConstUnicode pathName,        // IN:
    ret = utime(path, times);
 
    free(path);
+
    return ret;
 }
 
@@ -508,7 +578,7 @@ Posix_Utime(ConstUnicode pathName,        // IN:
  */
 
 void
-Posix_Perror(ConstUnicode str)         // IN:
+Posix_Perror(ConstUnicode str)  // IN:
 {
    char *tmpstr = Unicode_GetAllocBytes(str, STRING_ENCODING_DEFAULT);
 
@@ -519,7 +589,6 @@ Posix_Perror(ConstUnicode str)         // IN:
 }
 
 
-#if !defined(N_PLAT_NLM) // {
 /*
  *----------------------------------------------------------------------
  *
@@ -537,8 +606,8 @@ Posix_Perror(ConstUnicode str)         // IN:
  */
 
 long
-Posix_Pathconf(ConstUnicode pathName,   // IN:
-               int name)                // IN:
+Posix_Pathconf(ConstUnicode pathName,  // IN:
+               int name)               // IN:
 {
    char *path;
    long ret;
@@ -550,6 +619,7 @@ Posix_Pathconf(ConstUnicode pathName,   // IN:
    ret = pathconf(path, name);
 
    free(path);
+
    return ret;
 }
 
@@ -587,6 +657,7 @@ Posix_Popen(ConstUnicode pathName,  // IN:
    stream = popen(path, mode);
 
    free(path);
+
    return stream;
 }
 
@@ -623,6 +694,7 @@ Posix_Mknod(ConstUnicode pathName,  // IN:
    ret = mknod(path, mode, dev);
 
    free(path);
+
    return ret;
 }
 
@@ -659,6 +731,7 @@ Posix_Chown(ConstUnicode pathName,  // IN:
    ret = chown(path, owner, group);
 
    free(path);
+
    return ret;
 }
 
@@ -695,6 +768,7 @@ Posix_Lchown(ConstUnicode pathName,  // IN:
    ret = lchown(path, owner, group);
 
    free(path);
+
    return ret;
 }
 
@@ -729,6 +803,7 @@ Posix_Link(ConstUnicode pathName1,  // IN:
    }
    if (!PosixConvertToCurrent(pathName2, &path2)) {
       free(path1);
+
       return -1;
    }
 
@@ -736,6 +811,7 @@ Posix_Link(ConstUnicode pathName1,  // IN:
 
    free(path1);
    free(path2);
+
    return ret;
 }
 
@@ -770,6 +846,7 @@ Posix_Symlink(ConstUnicode pathName1,  // IN:
    }
    if (!PosixConvertToCurrent(pathName2, &path2)) {
       free(path1);
+
       return -1;
    }
 
@@ -777,6 +854,7 @@ Posix_Symlink(ConstUnicode pathName1,  // IN:
 
    free(path1);
    free(path2);
+
    return ret;
 }
 
@@ -812,6 +890,7 @@ Posix_Mkfifo(ConstUnicode pathName,  // IN:
    ret = mkfifo(path, mode);
 
    free(path);
+
    return ret;
 }
 
@@ -847,6 +926,7 @@ Posix_Truncate(ConstUnicode pathName,  // IN:
    ret = truncate(path, length);
 
    free(path);
+
    return ret;
 }
 
@@ -882,6 +962,7 @@ Posix_Utimes(ConstUnicode pathName,        // IN:
    ret = utimes(path, times);
 
    free(path);
+
    return ret;
 }
 
@@ -934,16 +1015,21 @@ Posix_Execl(ConstUnicode pathName,   // IN:
    if (argv) {
       errno = 0;
       if (count > 0) {
-	 PosixConvertToCurrent(arg0, &argv[0]);
+         if (!PosixConvertToCurrent(arg0, &argv[0])) {
+            goto exit;
+         }
          va_start(vl, arg0);
          for (i = 1; i < count; i++) {
-	    PosixConvertToCurrent(va_arg(vl, char *), &argv[i]);
+            if (!PosixConvertToCurrent(va_arg(vl, char *), &argv[i])) {
+               va_end(vl);
+               goto exit;
+            }
          }
          va_end(vl);
       }
       argv[count] = NULL;
       if (errno != 0) {
-	 goto exit;
+         goto exit;
       }
    }
 
@@ -951,9 +1037,10 @@ Posix_Execl(ConstUnicode pathName,   // IN:
 
 exit:
    if (argv) {
-      Util_FreeStringList(argv, count + 1);
+      Util_FreeStringList(argv, -1);
    }
    free(path);
+
    return ret;
 }
 
@@ -976,8 +1063,9 @@ exit:
  */
 
 int
-Posix_Execlp(ConstUnicode fileName,   // IN:
-             ConstUnicode arg0, ...)  // IN:
+Posix_Execlp(ConstUnicode fileName,  // IN:
+             ConstUnicode arg0,      // IN:
+             ...)                    // IN:
 {
    int ret = -1;
    char *file;
@@ -1006,16 +1094,21 @@ Posix_Execlp(ConstUnicode fileName,   // IN:
    if (argv) {
       errno = 0;
       if (count > 0) {
-	 PosixConvertToCurrent(arg0, &argv[0]);
+         if (!PosixConvertToCurrent(arg0, &argv[0])) {
+            goto exit;
+         }
          va_start(vl, arg0);
          for (i = 1; i < count; i++) {
-	    PosixConvertToCurrent(va_arg(vl, char *), &argv[i]);
+            if (!PosixConvertToCurrent(va_arg(vl, char *), &argv[i])) {
+               va_end(vl);
+               goto exit;
+            }
          }
          va_end(vl);
       }
       argv[count] = NULL;
       if (errno != 0) {
-	 goto exit;
+         goto exit;
       }
    }
 
@@ -1023,9 +1116,10 @@ Posix_Execlp(ConstUnicode fileName,   // IN:
 
 exit:
    if (argv) {
-      Util_FreeStringList(argv, count + 1);
+      Util_FreeStringList(argv, -1);
    }
    free(file);
+
    return ret;
 }
 
@@ -1048,8 +1142,8 @@ exit:
  */
 
 int
-Posix_Execv(ConstUnicode pathName,        // IN:
-            Unicode const argVal[])       // IN:
+Posix_Execv(ConstUnicode pathName,   // IN:
+            Unicode const argVal[])  // IN:
 {
    int ret = -1;
    char *path;
@@ -1069,6 +1163,7 @@ exit:
       Util_FreeStringList(argv, -1);
    }
    free(path);
+
    return ret;
 }
 
@@ -1091,9 +1186,9 @@ exit:
  */
 
 int
-Posix_Execve(ConstUnicode pathName,        // IN:
-	     Unicode const argVal[],       // IN:
-             Unicode const envPtr[])       // IN:
+Posix_Execve(ConstUnicode pathName,   // IN:
+	     Unicode const argVal[],  // IN:
+             Unicode const envPtr[])  // IN:
 {
    int ret = -1;
    char *path;
@@ -1120,6 +1215,7 @@ exit:
       Util_FreeStringList(envp, -1);
    }
    free(path);
+
    return ret;
 }
 
@@ -1142,8 +1238,8 @@ exit:
  */
 
 int
-Posix_Execvp(ConstUnicode fileName,        // IN:
-             Unicode const argVal[])       // IN:
+Posix_Execvp(ConstUnicode fileName,   // IN:
+             Unicode const argVal[])  // IN:
 {
    int ret = -1;
    char *file;
@@ -1163,6 +1259,7 @@ exit:
       Util_FreeStringList(argv, -1);
    }
    free(file);
+
    return ret;
 }
 
@@ -1184,7 +1281,7 @@ exit:
  */
 
 int
-Posix_System(ConstUnicode command)         // IN:
+Posix_System(ConstUnicode command)  // IN:
 {
    char *tmpcommand;
    int ret;
@@ -1196,6 +1293,7 @@ Posix_System(ConstUnicode command)         // IN:
    ret = system(tmpcommand);
 
    free(tmpcommand);
+
    return ret;
 }
 
@@ -1231,6 +1329,7 @@ Posix_Mkdir(ConstUnicode pathName,  // IN:
    ret = mkdir(path, mode);
 
    free(path);
+
    return ret;
 }
 
@@ -1265,6 +1364,7 @@ Posix_Chdir(ConstUnicode pathName)  // IN:
    ret = chdir(path);
 
    free(path);
+
    return ret;
 }
 
@@ -1300,6 +1400,7 @@ Posix_RealPath(ConstUnicode pathName)  // IN:
    p = realpath(path, rpath);
 
    free(path);
+
    return p == NULL ? NULL : Unicode_Alloc(rpath, STRING_ENCODING_DEFAULT);
 }
 
@@ -1324,26 +1425,35 @@ Posix_RealPath(ConstUnicode pathName)  // IN:
 Unicode
 Posix_ReadLink(ConstUnicode pathName)  // IN:
 {
-   char *path;
-   ssize_t bytes;
+   char *path = NULL;
    Unicode result = NULL;
-   char link[PATH_MAX];
 
-   if (!PosixConvertToCurrent(pathName, &path)) {
-      return NULL;
+   if (PosixConvertToCurrent(pathName, &path)) {
+      size_t size = 2 * 1024;
+
+      while (TRUE) {
+         char *linkPath = Util_SafeMalloc(size);
+         ssize_t len = readlink(path, linkPath, size);
+
+         if (len == -1) {
+            free(linkPath);
+            break;
+         }
+
+         if (len < size) {
+            linkPath[len] = '\0'; // Add the missing NUL to path
+            result = Unicode_Alloc(linkPath, STRING_ENCODING_DEFAULT);
+            free(linkPath);
+            break;
+         }
+         free(linkPath);
+
+         size += 1024;
+      }
    }
 
-   bytes = readlink(path, link, sizeof link - 1);
-   ASSERT_NOT_IMPLEMENTED(bytes < (ssize_t) sizeof link);
-
    free(path);
 
-   if (bytes != -1) {
-      /* add the missing NUL character to path */
-      link[bytes] = '\0';
-      result = Unicode_Alloc(link, STRING_ENCODING_DEFAULT);
-   }
-
    return result;
 }
 
@@ -1379,6 +1489,7 @@ Posix_Lstat(ConstUnicode pathName,  // IN:
    ret = lstat(path, statbuf);
 
    free(path);
+
    return ret;
 }
 
@@ -1413,6 +1524,7 @@ Posix_OpenDir(ConstUnicode pathName)  // IN:
    ret = opendir(path);
 
    free(path);
+
    return ret;
 }
 
@@ -1475,9 +1587,10 @@ Posix_Getenv(ConstUnicode name)  // IN:
  */
 
 int
-Posix_Putenv(Unicode name)   // IN:
+Posix_Putenv(Unicode name)  // IN:
 {
    ASSERT(Unicode_IsBufferValid(name, -1, STRING_ENCODING_US_ASCII));
+
    return putenv(name);
 }
 
@@ -1533,9 +1646,8 @@ Posix_Getpwnam(ConstUnicode name)  // IN:
 struct passwd *
 Posix_Getpwuid(uid_t uid)  // IN:
 {
-   struct passwd *pw;
+   struct passwd *pw = getpwuid(uid);
 
-   pw = getpwuid(uid);
    return GetpwInternal(pw);
 }
 
@@ -1573,8 +1685,10 @@ GetpwInternal(struct passwd *pw)  // IN:
    spw.pw_dir = NULL;
    free(spw.pw_name);
    spw.pw_name = NULL;
+#if !defined __ANDROID__
    free(spw.pw_gecos);
    spw.pw_gecos = NULL;
+#endif
    free(spw.pw_shell);
    spw.pw_shell = NULL;
 #if defined(__FreeBSD__)
@@ -1597,28 +1711,36 @@ GetpwInternal(struct passwd *pw)  // IN:
    ret = EIO;
 #endif
    if (pw->pw_passwd &&
-       (spw.pw_passwd = Unicode_Alloc(pw->pw_passwd, STRING_ENCODING_DEFAULT)) == NULL) {
+       (spw.pw_passwd = Unicode_Alloc(pw->pw_passwd,
+                                      STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (pw->pw_dir &&
-       (spw.pw_dir = Unicode_Alloc(pw->pw_dir, STRING_ENCODING_DEFAULT)) == NULL) {
+       (spw.pw_dir = Unicode_Alloc(pw->pw_dir,
+                                   STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (pw->pw_name &&
-       (spw.pw_name = Unicode_Alloc(pw->pw_name, STRING_ENCODING_DEFAULT)) == NULL) {
+       (spw.pw_name = Unicode_Alloc(pw->pw_name,
+                                    STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#if !defined __ANDROID__
    if (pw->pw_gecos &&
-       (spw.pw_gecos = Unicode_Alloc(pw->pw_gecos, STRING_ENCODING_DEFAULT)) == NULL) {
+       (spw.pw_gecos = Unicode_Alloc(pw->pw_gecos,
+                                     STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#endif
    if (pw->pw_shell &&
-       (spw.pw_shell = Unicode_Alloc(pw->pw_shell, STRING_ENCODING_DEFAULT)) == NULL) {
+       (spw.pw_shell = Unicode_Alloc(pw->pw_shell,
+                                     STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
 #if defined(__FreeBSD__)
    if (pw->pw_class &&
-       (spw.pw_class = Unicode_Alloc(pw->pw_class, STRING_ENCODING_DEFAULT)) == NULL) {
+       (spw.pw_class = Unicode_Alloc(pw->pw_class,
+                                     STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
 #endif
@@ -1666,6 +1788,7 @@ Posix_Statfs(ConstUnicode pathName,     // IN:
    ret = statfs(path, statfsbuf);
 
    free(path);
+
    return ret;
 }
 #endif // } !defined(sun)
@@ -1716,11 +1839,12 @@ Posix_Setenv(ConstUnicode name,   // IN:
       if (!trackEnv) {
          trackEnv = HashTable_Alloc(16, HASH_STRING_KEY, free);
       }
+
       /*
-       * In order to keep memory management and hash table manipulation simple, each env
-       * var is stored as a memory block containing the NUL-terminated environment
-       * variable name, followed immediately in memory by the full argument to putenv
-       * ('varname=value').
+       * In order to keep memory management and hash table manipulation simple,
+       * each env var is stored as a memory block containing the NUL-terminated
+       * environment variable name, followed immediately in memory by the
+       * full argument to putenv ('varname=value').
        */
 
       rawNameLen = strlen(rawName) + 1;
@@ -1728,9 +1852,12 @@ Posix_Setenv(ConstUnicode name,   // IN:
       fslen = rawNameLen + rawValueLen + 1; // 1 is for '=' sign
       keyStr = malloc(rawNameLen + fslen);
       fullStr = keyStr + rawNameLen;
+
       /*
-       * Use memcpy because Str_Snprintf() doesn't play well with non-UTF8 strings.
+       * Use memcpy because Str_Snprintf() doesn't play well with non-UTF8
+       * strings.
        */
+
       memcpy(keyStr, rawName, rawNameLen);
       memcpy(fullStr, rawName, rawNameLen);
       fullStr[rawNameLen - 1] = '=';
@@ -1748,6 +1875,7 @@ Posix_Setenv(ConstUnicode name,   // IN:
 exit:
    free(rawName);
    free(rawValue);
+
    return ret;
 }
 
@@ -1769,7 +1897,7 @@ exit:
  */
 
 void
-Posix_Unsetenv(ConstUnicode name)   // IN:
+Posix_Unsetenv(ConstUnicode name)  // IN:
 {
    char *rawName;
 
@@ -1803,10 +1931,15 @@ Posix_Unsetenv(ConstUnicode name)   // IN:
 struct passwd *
 Posix_Getpwent(void)
 {
-   struct passwd *pw;
+#if defined NO_GETPWENT
+   NOT_IMPLEMENTED();
+   errno = ENOSYS;
+   return NULL;
+#else
+   struct passwd *pw = getpwent();
 
-   pw = getpwent();
    return GetpwInternal(pw);
+#endif
 }
 
 #if !defined(VM_SYSTEM_HAS_GETPWNAM_R) || \
@@ -1817,8 +1950,9 @@ Posix_Getpwent(void)
  *
  * CopyFieldIntoBuf --
  *
- *      Copies a field in a passwd/group structure into the supplied buffer, and sets
- *      that pointer into dest. Used as a helper function for the EmulateGet* routines.
+ *      Copies a field in a passwd/group structure into the supplied buffer,
+ *      and sets that pointer into dest. Used as a helper function for the
+ *      EmulateGet* routines.
  *
  * Results:
  *      TRUE on success, FALSE on failure.
@@ -1830,10 +1964,10 @@ Posix_Getpwent(void)
  */
 
 static Bool
-CopyFieldIntoBuf(const char *src,
-                 char **dest,
-                 char **buf,
-                 size_t *bufLen)
+CopyFieldIntoBuf(const char *src,  // IN:
+                 char **dest,      // OUT:
+                 char **buf,       // OUT
+                 size_t *bufLen)   // OUT:
 {
    if (src) {
       size_t needLen = strlen(src) + 1;
@@ -1892,9 +2026,11 @@ PasswdCopy(struct passwd *orig, // IN
    if (!CopyFieldIntoBuf(orig->pw_passwd, &new->pw_passwd, &buf, &bufLen)) {
       return NULL;
    }
+#if !defined __ANDROID__
    if (!CopyFieldIntoBuf(orig->pw_gecos, &new->pw_gecos, &buf, &bufLen)) {
       return NULL;
    }
+#endif
    if (!CopyFieldIntoBuf(orig->pw_dir, &new->pw_dir, &buf, &bufLen)) {
       return NULL;
    }
@@ -1985,11 +2121,11 @@ EmulateGetpwnam_r(const char *name,       // IN
  */
 
 static int
-EmulateGetpwuid_r(uid_t uid,              // IN
-                  struct passwd *pwbuf,   // IN/OUT
-                  char *buf,              // IN
-                  size_t buflen,          // IN
-                  struct passwd **pwbufp) // IN/OUT
+EmulateGetpwuid_r(uid_t uid,               // IN:
+                  struct passwd *pwbuf,    // IN/OUT:
+                  char *buf,               // IN:
+                  size_t buflen,           // IN:
+                  struct passwd **pwbufp)  // IN/OUT:
 {
    static Atomic_uint32 mutex = {0};
    struct passwd *pw;
@@ -2002,6 +2138,7 @@ EmulateGetpwuid_r(uid_t uid,              // IN
    /*
     * XXX Use YIELD() here when it works on FreeBSD.
     */
+
    while (Atomic_ReadWrite(&mutex, 1)); // Spinlock
 
    pw = getpwuid(uid);
@@ -2039,10 +2176,10 @@ EmulateGetpwuid_r(uid_t uid,              // IN
  */
 
 static struct group *
-GroupCopy(struct group *orig, // IN
-          struct group *new,  // IN/OUT
-          char *buf,          // IN
-          size_t bufLen)      // IN
+GroupCopy(struct group *orig,  // IN:
+          struct group *new,   // IN/OUT:
+          char *buf,           // IN:
+          size_t bufLen)       // IN:
 {
    if (!orig) {
       return NULL;
@@ -2063,10 +2200,12 @@ GroupCopy(struct group *orig, // IN
       char **newGrMem;
 
       /*
-       * Before putting the gr_mem 'char **' array into 'buf', aligns the buffer to a
-       * pointer-size boundary.
+       * Before putting the gr_mem 'char **' array into 'buf', aligns the
+       * buffer to a pointer-size boundary.
        */
-      alignLen = ((((uintptr_t) buf) + (sizeof(void *) - 1)) & ~(sizeof(void *) - 1));
+
+      alignLen = ((((uintptr_t) buf) +
+                    (sizeof(void *) - 1)) & ~(sizeof(void *) - 1));
       alignLen -= ((uintptr_t) buf);
 
       if (bufLen < alignLen) {
@@ -2078,6 +2217,7 @@ GroupCopy(struct group *orig, // IN
       /*
        * Count the number of items in the gr_mem array, and then copy them all.
        */
+
       for (i = 0; orig->gr_mem[i]; i++);
       i++; // need space for a terminating NULL
 
@@ -2127,11 +2267,11 @@ GroupCopy(struct group *orig, // IN
  */
 
 static int
-EmulateGetgrnam_r(const char *name,       // IN
-                  struct group *grbuf,    // IN/OUT
-                  char *buf,              // IN
-                  size_t buflen,          // IN
-                  struct group **grbufp)  // IN/OUT
+EmulateGetgrnam_r(const char *name,       // IN:
+                  struct group *grbuf,    // IN/OUT:
+                  char *buf,              // IN:
+                  size_t buflen,          // IN:
+                  struct group **grbufp)  // IN/OUT:
 {
    static Atomic_uint32 mutex = {0};
    struct group *gr;
@@ -2145,6 +2285,7 @@ EmulateGetgrnam_r(const char *name,       // IN
    /*
     * XXX Use YIELD() here once it is available on FreeBSD
     */
+
    while (Atomic_ReadWrite(&mutex, 1)); // Spinlock
 
    gr = getgrnam(name);
@@ -2199,6 +2340,7 @@ Posix_Getpwnam_r(ConstUnicode name,    // IN:
        */
 
       *ppw = NULL;
+
       return errno;
    }
 
@@ -2251,7 +2393,7 @@ Posix_Getpwuid_r(uid_t uid,            // IN:
    ret = EmulateGetpwuid_r(uid, pw, buf, size, ppw);
 #endif
    if (ret != 0 || *ppw == NULL) {
-   // ret is errno on failure, *ppw is NULL if no matching entry found.
+      // ret is errno on failure, *ppw is NULL if no matching entry found.
       return ret;
    }
 
@@ -2285,7 +2427,9 @@ GetpwInternal_r(struct passwd *pw,    // IN:
    int ret;
    char *pwname = NULL;
    char *passwd = NULL;
+#if !defined __ANDROID__
    char *gecos = NULL;
+#endif
    char *dir = NULL;
    char *shell = NULL;
    size_t n;
@@ -2303,23 +2447,30 @@ GetpwInternal_r(struct passwd *pw,    // IN:
 
    ret = ENOMEM;
    if (pw->pw_name &&
-       (pwname = Unicode_Alloc(pw->pw_name, STRING_ENCODING_DEFAULT)) == NULL) {
+       (pwname = Unicode_Alloc(pw->pw_name,
+                               STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (pw->pw_passwd &&
-       (passwd = Unicode_Alloc(pw->pw_passwd, STRING_ENCODING_DEFAULT)) == NULL) {
+       (passwd = Unicode_Alloc(pw->pw_passwd,
+                               STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#if !defined __ANDROID__
    if (pw->pw_gecos &&
-       (gecos = Unicode_Alloc(pw->pw_gecos, STRING_ENCODING_DEFAULT)) == NULL) {
+       (gecos = Unicode_Alloc(pw->pw_gecos,
+                              STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
+#endif
    if (pw->pw_dir &&
-       (dir = Unicode_Alloc(pw->pw_dir, STRING_ENCODING_DEFAULT)) == NULL) {
+       (dir = Unicode_Alloc(pw->pw_dir,
+                            STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (pw->pw_shell &&
-       (shell = Unicode_Alloc(pw->pw_shell, STRING_ENCODING_DEFAULT)) == NULL) {
+       (shell = Unicode_Alloc(pw->pw_shell,
+                              STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
 
@@ -2332,6 +2483,7 @@ GetpwInternal_r(struct passwd *pw,    // IN:
 
    if (pwname) {
       size_t len = strlen(pwname) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2341,24 +2493,27 @@ GetpwInternal_r(struct passwd *pw,    // IN:
 
    if (passwd != NULL) {
       size_t len = strlen(passwd) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
       pw->pw_passwd = memcpy(buf + n, passwd, len);
       n += len;
    }
-
+#if !defined __ANDROID__
    if (gecos) {
       size_t len = strlen(gecos) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
       pw->pw_gecos = memcpy(buf + n, gecos, len);
       n += len;
    }
-
+#endif
    if (dir) {
       size_t len = strlen(dir) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2368,6 +2523,7 @@ GetpwInternal_r(struct passwd *pw,    // IN:
 
    if (shell) {
       size_t len = strlen(shell) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2380,8 +2536,11 @@ exit:
    free(passwd);
    free(dir);
    free(pwname);
+#if !defined __ANDROID__
    free(gecos);
+#endif
    free(shell);
+
    return ret;
 }
 
@@ -2406,10 +2565,10 @@ exit:
  */
 
 int
-Posix_GetGroupList(ConstUnicode user,    // IN:
-                   gid_t group,          // IN:
-                   gid_t *groups,        // OUT:
-                   int *ngroups)         // IN/OUT:
+Posix_GetGroupList(ConstUnicode user,  // IN:
+                   gid_t group,        // IN:
+                   gid_t *groups,      // OUT:
+                   int *ngroups)       // IN/OUT:
 {
    char *tmpuser;
    int ret;
@@ -2435,12 +2594,14 @@ Posix_GetGroupList(ConstUnicode user,    // IN:
       }
       ASSERT(groups != NULL);
       *groups = group;
+
       return 1;
    }
 
    ret = getgrouplist(tmpuser, group, groups, ngroups);
 
    free(tmpuser);
+
    return ret;
 }
 
@@ -2495,15 +2656,18 @@ Posix_Getgrnam(ConstUnicode name)  // IN:
 
    ret = ENOMEM;
    if (gr->gr_passwd &&
-       (sgr.gr_passwd = Unicode_Alloc(gr->gr_passwd, STRING_ENCODING_DEFAULT)) == NULL) {
+       (sgr.gr_passwd = Unicode_Alloc(gr->gr_passwd,
+                                      STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (gr->gr_name &&
-       (sgr.gr_name = Unicode_Alloc(gr->gr_name, STRING_ENCODING_DEFAULT)) == NULL) {
+       (sgr.gr_name = Unicode_Alloc(gr->gr_name,
+                                    STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (gr->gr_mem) {
-      sgr.gr_mem = Unicode_AllocList(gr->gr_mem, -1, STRING_ENCODING_DEFAULT);
+      sgr.gr_mem = Unicode_AllocList(gr->gr_mem, -1,
+                                     STRING_ENCODING_DEFAULT);
    }
 
    ret = 0;
@@ -2513,8 +2677,8 @@ Posix_Getgrnam(ConstUnicode name)  // IN:
       errno = ret;
       return NULL;
    }
-   return &sgr;
 
+   return &sgr;
 }
 
 
@@ -2557,6 +2721,7 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
        */
 
       *pgr = NULL;
+
       return errno;
    }
 
@@ -2585,11 +2750,13 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
 
    ret = ENOMEM;
    if (gr->gr_name &&
-       (grname = Unicode_Alloc(gr->gr_name, STRING_ENCODING_DEFAULT)) == NULL) {
+       (grname = Unicode_Alloc(gr->gr_name,
+                               STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (gr->gr_passwd &&
-       (grpasswd = Unicode_Alloc(gr->gr_passwd, STRING_ENCODING_DEFAULT)) == NULL) {
+       (grpasswd = Unicode_Alloc(gr->gr_passwd,
+                                 STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (gr->gr_mem) {
@@ -2605,6 +2772,7 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
 
    if (grname) {
       size_t len = strlen(grname) + 1;
+
       if (n + len > size) {
          goto exit;
       }
@@ -2614,6 +2782,7 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
 
    if (grpasswd != NULL) {
       size_t len = strlen(grpasswd) + 1;
+
       if (n + len > size) {
          goto exit;
       }
@@ -2624,6 +2793,7 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
    if (grmem) {
       for (i = 0; grmem[i]; i++) {
          size_t len = strlen(grmem[i]) + 1;
+
          if (n + len > size) {
 	    goto exit;
          }
@@ -2640,6 +2810,7 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
    if (grmem) {
       Unicode_FreeList(grmem, -1);
    }
+
    return ret;
 }
 
@@ -2665,11 +2836,11 @@ Posix_Getgrnam_r(ConstUnicode name,    // IN:
  */
 
 int
-Posix_Mount(ConstUnicode source,             // IN:
-            ConstUnicode target,             // IN:
-            const char *filesystemtype,      // IN:
-            unsigned long mountflags,        // IN:
-            const void *data)                // IN:
+Posix_Mount(ConstUnicode source,         // IN:
+            ConstUnicode target,         // IN:
+            const char *filesystemtype,  // IN:
+            unsigned long mountflags,    // IN:
+            const void *data)            // IN:
 {
    int ret = -1;
    char *tmpsource = NULL;
@@ -2687,6 +2858,7 @@ Posix_Mount(ConstUnicode source,             // IN:
 exit:
    free(tmpsource);
    free(tmptarget);
+
    return ret;
 }
 
@@ -2709,7 +2881,7 @@ exit:
  */
 
 int
-Posix_Umount(ConstUnicode target)             // IN:
+Posix_Umount(ConstUnicode target)  // IN:
 {
    char *tmptarget;
    int ret;
@@ -2721,6 +2893,7 @@ Posix_Umount(ConstUnicode target)             // IN:
    ret = umount(tmptarget);
 
    free(tmptarget);
+
    return ret;
 }
 
@@ -2743,9 +2916,14 @@ Posix_Umount(ConstUnicode target)             // IN:
  */
 
 FILE *
-Posix_Setmntent(ConstUnicode pathName, // IN:
-                const char *mode)      // IN:
+Posix_Setmntent(ConstUnicode pathName,  // IN:
+                const char *mode)       // IN:
 {
+#if defined NO_SETMNTENT
+   NOT_IMPLEMENTED();
+   errno = ENOSYS;
+   return NULL;
+#else
    char *path;
    FILE *stream;
 
@@ -2754,11 +2932,11 @@ Posix_Setmntent(ConstUnicode pathName, // IN:
    if (!PosixConvertToCurrent(pathName, &path)) {
       return NULL;
    }
-
    stream = setmntent(path, mode);
-
    free(path);
+
    return stream;
+#endif
 }
 
 
@@ -2806,19 +2984,23 @@ Posix_Getmntent(FILE *fp)  // IN:
 
    ret = ENOMEM;
    if (m->mnt_fsname &&
-       (sm.mnt_fsname = Unicode_Alloc(m->mnt_fsname, STRING_ENCODING_DEFAULT)) == NULL) {
+       (sm.mnt_fsname = Unicode_Alloc(m->mnt_fsname,
+                                      STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (m->mnt_dir &&
-       (sm.mnt_dir = Unicode_Alloc(m->mnt_dir, STRING_ENCODING_DEFAULT)) == NULL) {
+       (sm.mnt_dir = Unicode_Alloc(m->mnt_dir,
+                                   STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (m->mnt_type &&
-       (sm.mnt_type = Unicode_Alloc(m->mnt_type, STRING_ENCODING_DEFAULT)) == NULL) {
+       (sm.mnt_type = Unicode_Alloc(m->mnt_type,
+                                    STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (m->mnt_opts &&
-       (sm.mnt_opts = Unicode_Alloc(m->mnt_opts, STRING_ENCODING_DEFAULT)) == NULL) {
+       (sm.mnt_opts = Unicode_Alloc(m->mnt_opts,
+                                    STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    ret = 0;
@@ -2828,6 +3010,7 @@ exit:
       errno = ret;
       return NULL;
    }
+
    return &sm;
 }
 
@@ -2854,6 +3037,11 @@ Posix_Getmntent_r(FILE *fp,          // IN:
                   char *buf,         // IN:
                   int size)          // IN:
 {
+#if defined NO_GETMNTENT_R
+   NOT_IMPLEMENTED();
+   errno = ENOSYS;
+   return NULL;
+#else
    int ret;
    char *fsname = NULL;
    char *dir = NULL;
@@ -2871,15 +3059,18 @@ Posix_Getmntent_r(FILE *fp,          // IN:
 
    ret = ENOMEM;
    if (m->mnt_fsname &&
-       (fsname = Unicode_Alloc(m->mnt_fsname, STRING_ENCODING_DEFAULT)) == NULL) {
+       (fsname = Unicode_Alloc(m->mnt_fsname,
+                               STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (m->mnt_dir &&
-       (dir = Unicode_Alloc(m->mnt_dir, STRING_ENCODING_DEFAULT)) == NULL) {
+       (dir = Unicode_Alloc(m->mnt_dir,
+                            STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (m->mnt_type &&
-       (type = Unicode_Alloc(m->mnt_type, STRING_ENCODING_DEFAULT)) == NULL) {
+       (type = Unicode_Alloc(m->mnt_type,
+                             STRING_ENCODING_DEFAULT)) == NULL) {
       goto exit;
    }
    if (m->mnt_opts &&
@@ -2896,6 +3087,7 @@ Posix_Getmntent_r(FILE *fp,          // IN:
 
    if (fsname) {
       int len = strlen(fsname) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2905,6 +3097,7 @@ Posix_Getmntent_r(FILE *fp,          // IN:
 
    if (dir != NULL) {
       int len = strlen(dir) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2914,6 +3107,7 @@ Posix_Getmntent_r(FILE *fp,          // IN:
 
    if (type) {
       int len = strlen(type) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2923,6 +3117,7 @@ Posix_Getmntent_r(FILE *fp,          // IN:
 
    if (opts) {
       size_t len = strlen(opts) + 1;
+
       if (n + len > size || n + len < n) {
          goto exit;
       }
@@ -2940,9 +3135,12 @@ exit:
 
    if (ret != 0) {
       errno = ret;
+
       return NULL;
    }
+
    return m;
+#endif // defined __ANDROID__
 }
 
 
@@ -2964,8 +3162,8 @@ exit:
  */
 
 int
-Posix_Printf(ConstUnicode format,
-             ...)
+Posix_Printf(ConstUnicode format,  // IN:
+             ...)                  // IN:
 {
    va_list args;
    Unicode output;
@@ -2983,6 +3181,7 @@ Posix_Printf(ConstUnicode format,
 
    free(output);
    free(outCurr);
+
    return numChars;
 }
 
@@ -3005,9 +3204,9 @@ Posix_Printf(ConstUnicode format,
  */
 
 int
-Posix_Fprintf(FILE *stream,
-              ConstUnicode format,
-              ...)
+Posix_Fprintf(FILE *stream,         // IN:
+              ConstUnicode format,  // IN:
+              ...)                  // IN:
 {
    va_list args;
    Unicode output;
@@ -3025,6 +3224,7 @@ Posix_Fprintf(FILE *stream,
 
    free(output);
    free(outCurr);
+
    return nOutput;
 }
 
@@ -3079,6 +3279,45 @@ Posix_Getmntent(FILE *fp,           // IN:
 
    return ret;
 }
-
 #endif // } !defined(sun)
-#endif // } !defined(N_PLAT_NLM)
+
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Posix_MkTemp --
+ *
+ *      POSIX mktemp().  It is implemented via mkstemp() to avoid
+ *      warning about using dangerous mktemp() API - but note that
+ *      it suffers from all mktemp() problems - caller has to use
+ *      O_EXCL when creating file, and retry if file already exists.
+ *
+ * Results:
+ *      NULL    Error
+ *      !NULL   Success (result must be freed by the caller)
+ *
+ * Side effects:
+ *      errno is set on error
+ *
+ *----------------------------------------------------------------------
+ */
+
+Unicode
+Posix_MkTemp(ConstUnicode pathName)  // IN:
+{
+   Unicode result = NULL;
+   char *path;
+   int fd;
+
+   if (!PosixConvertToCurrent(pathName, &path)) {
+      return NULL;
+   }
+   fd = mkstemp(path);
+   if (fd >= 0) {
+      close(fd);
+      unlink(path);
+      result = Unicode_Alloc(path, STRING_ENCODING_DEFAULT);
+   }
+   free(path);
+   return result;
+}
