@@ -100,6 +100,7 @@
 #include "str.h"
 #include "util.h"
 #include "hashTable.h"
+#include "hostinfo.h"
 
 
 /*
@@ -119,20 +120,25 @@ typedef DWORD VThreadBaseKeyType;
 #define VTHREADBASE_INVALID_KEY (VThreadBaseKeyType)(TLS_OUT_OF_INDEXES)
 #else
 typedef pthread_key_t VThreadBaseKeyType;
+/* PTHREAD_KEYS_MAX not defined on Android. */
+#if defined __linux__ && !defined PTHREAD_KEYS_MAX
+#define VTHREADBASE_INVALID_KEY (VThreadBaseKeyType)(1024)
+#else
 #define VTHREADBASE_INVALID_KEY (VThreadBaseKeyType)(PTHREAD_KEYS_MAX)
 #endif
+#endif
 
 static void VThreadBaseSimpleNoID(void);
 static void VThreadBaseSimpleFreeID(void *tlsData);
 static void VThreadBaseSafeDeleteTLS(void *data);
 
 static struct {
-   Atomic_Int key;
-   Atomic_Int dynamicID;
-   Atomic_Int numThreads;
-   Atomic_Ptr nativeHash;
-   void (*noIDFunc)(void);
-   void (*freeIDFunc)(void *);
+   Atomic_Int   key;
+   Atomic_Int   dynamicID;
+   Atomic_Int   numThreads;
+   Atomic_Ptr   nativeHash;
+   void       (*noIDFunc)(void);
+   void       (*freeIDFunc)(void *);
 } vthreadBaseGlobals = {
    { VTHREADBASE_INVALID_KEY },
    { VTHREAD_ALLOCSTART_ID },
@@ -205,7 +211,7 @@ static struct {
  */
 
 #if defined __GNUC__
-/* gcc-4.1.0 and gcc-4.1.1 have buggy weak-symbol optimiziation. */
+/* gcc-4.1.0 and gcc-4.1.1 have buggy weak-symbol optimization. */
 #  if __GNUC__ == 4 && __GNUC_MINOR__ == 1 && \
      (__GNUC_PATCHLEVEL__ == 0 || __GNUC_PATCHLEVEL__ == 1)
 #  error Cannot build VThreadBase with weak symbols: buggy gcc version
@@ -394,6 +400,15 @@ VThreadBaseGetKey(void)
 #else
       Bool success = pthread_key_create(&newKey, 
                                         &VThreadBaseSafeDeleteTLS) == 0;
+      if (success && newKey == 0) {
+         /* 
+          * Leak TLS key 0.  System libraries have a habit of destroying
+          * it.  See bugs 702818 and 773420.
+          */
+
+         success = pthread_key_create(&newKey, 
+                                      &VThreadBaseSafeDeleteTLS) == 0;
+      }
       ASSERT_NOT_IMPLEMENTED(success);
 #endif
 
@@ -914,19 +929,27 @@ VThreadBaseGetNative(void)
 static Bool
 VThreadBaseNativeIsAlive(void *native)
 {
-   HANDLE hThread = OpenThread(THREAD_QUERY_INFORMATION, FALSE,
+   // Different access level due to impersonation see PR#780775
+   HANDLE hThread = OpenThread(Hostinfo_OpenThreadBits(), FALSE,
                                (DWORD)(uintptr_t)native);
 
-   if (hThread != NULL) {
+   if (hThread == NULL) {
+      /*
+       * An access denied error tells us that the process is alive despite
+       * the inability of accessing its information. Commonly, access denied
+       * occurs when a process is trying to completely protect itself (e.g.
+       * a virus checker).
+       */
+
+      return (GetLastError() == ERROR_ACCESS_DENIED) ? TRUE : FALSE;
+   } else {
       DWORD exitCode;
-      BOOL success;
+      BOOL success = GetExitCodeThread(hThread, &exitCode);
 
-      success = GetExitCodeThread(hThread, &exitCode);
       ASSERT(success);  /* No known ways GetExitCodeThread can fail */
       CloseHandle(hThread);
+
       return exitCode == STILL_ACTIVE;
-   } else {
-      return FALSE;
    }
 }
 #endif
@@ -998,7 +1021,7 @@ VThreadBaseSimpleNoID(void)
 
       newID = Atomic_FetchAndInc(&vthreadBaseGlobals.dynamicID);
       /*
-       * Detect VThreadID overflow (~0 is used as a sentinal).
+       * Detect VThreadID overflow (~0 is used as a sentinel).
        * Leave a space of ~10 IDs, since the increment and bounds-check
        * are not atomic.
        */
