@@ -61,7 +61,8 @@ ExtractArgStringFromAddressSpaceFile(FileIODescriptor asFd,
                                      uintptr_t offset);
 
 static char *
-ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo);
+ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo,
+                                       char **procCmdName);
 
 /*
  *----------------------------------------------------------------------------
@@ -85,43 +86,39 @@ ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo);
  *----------------------------------------------------------------------------
  */
 
-ProcMgr_ProcList *
+ProcMgrProcInfoArray *
 ProcMgr_ListProcesses(void)
 {
-   ProcMgr_ProcList *procList = NULL;
-
-   Bool failed = FALSE;
-   DynBuf dbProcId;
-   DynBuf dbProcCmd;
-   DynBuf dbProcStartTime;
-   DynBuf dbProcOwner;
+   ProcMgrProcInfoArray *procList = NULL;
+   ProcMgrProcInfo processInfo;
+   Bool failed = TRUE;
    DIR *dir;
    struct dirent *ent;
 
-   DynBuf_Init(&dbProcId);
-   DynBuf_Init(&dbProcCmd);
-   DynBuf_Init(&dbProcStartTime);
-   DynBuf_Init(&dbProcOwner);
+   procList = Util_SafeCalloc(1, sizeof *procList);
+   ProcMgrProcInfoArray_Init(procList, 0);
+   processInfo.procOwner = NULL;
+   processInfo.procCmdLine = NULL;
+   processInfo.procCmdName = NULL;
 
    dir = opendir("/proc");
    if (NULL == dir) {
       Warning("ProcMgr_ListProcesses unable to open /proc\n");
-      failed = TRUE;
       goto exit;
    }
 
    while (TRUE) {
-      pid_t pid;
+      char *tmp;
+      char *cmdNameBegin = NULL;
+      char *cmdNameEnd = NULL;
       struct passwd *pwd;
-      char *cmdLineTemp;
-      char *userName = NULL;
       char tempPath[MAXPATHLEN];
-      time_t processStartTime;
       psinfo_t procInfo;
       size_t strLen = 0;
       size_t numRead = 0;
       FileIODescriptor psInfoFd;
       FileIOResult res;
+      Bool cmdNameLookup = TRUE;
 
       errno = 0;
       FileIO_Invalidate(&psInfoFd);
@@ -131,7 +128,6 @@ ProcMgr_ListProcesses(void)
          if (errno == 0) {
             break;
          } else {
-            failed = TRUE;
             goto exit;
          }
       }
@@ -152,7 +148,6 @@ ProcMgr_ListProcesses(void)
              (res == FILEIO_NO_PERMISSION)) {
             continue;
          } else {
-            failed = TRUE;
             goto exit;
          }
       }
@@ -160,101 +155,120 @@ ProcMgr_ListProcesses(void)
       res = FileIO_Read(&psInfoFd, &procInfo, sizeof procInfo, &numRead);
       FileIO_Close(&psInfoFd);
       if (res != FILEIO_SUCCESS) {
-         failed = TRUE;
          goto exit;
       }
 
-      processStartTime = procInfo.pr_start.tv_sec;
+      processInfo.procStartTime = procInfo.pr_start.tv_sec;
 
       /*
-       * Command line strings in procInfo.pr_psargs are truncated to PRARGZ
-       * bytes. In this case we extract the arguments from the /proc/<pid>/as
-       * file. Since most command line strings are expected to fit within
-       * PRARGSZ bytes, we avoid calling
-       * ExtractCommandLineFromAddressSpaceFile for every process.
+       * If the command name in the ps info struct is strictly less than the
+       * maximum allowed size, then we can save it right now. Else we shall
+       * need to try and parse it from the entire command line, to avoid
+       * saving a truncated command name.
        */
-      if (strlen(procInfo.pr_psargs) + 1 == PRARGSZ) {
-         char *tmp;
-
-         tmp = ExtractCommandLineFromAddressSpaceFile(&procInfo);
-         if (tmp != NULL) {
-            cmdLineTemp = Unicode_Alloc(tmp, STRING_ENCODING_DEFAULT);
-            free(tmp);
-         } else {
-            cmdLineTemp = Unicode_Alloc(procInfo.pr_psargs,
-                                        STRING_ENCODING_DEFAULT);
-         }
-      } else {
-         cmdLineTemp = Unicode_Alloc(procInfo.pr_psargs,
-                                     STRING_ENCODING_DEFAULT);
+      if (strlen(procInfo.pr_fname) + 1 < sizeof procInfo.pr_fname) {
+         processInfo.procCmdName = Unicode_Alloc(procInfo.pr_fname,
+                                                 STRING_ENCODING_DEFAULT);
+         cmdNameLookup = FALSE;
       }
 
       /*
-       * Store the command line string pointer in dynbuf.
+       * The logic below is this:
+       * 1. If we are looking for the explicit command name, we need to
+       *    extract the arguments from the /proc/<pid>/as file and save argv[0].
+       * 2. If we are not looking for the explicit command name, but the command
+       *    line in the ps info struct is not strictly less than the maximum
+       *    allowed size, we still need to extract the arguments from the
+       *    /proc/<pid>/as file, to avoid saving truncated comand line.
+       * 3. Else we can save the command line directly from the ps info struct.
        */
-      DynBuf_Append(&dbProcCmd, &cmdLineTemp, sizeof cmdLineTemp);
+      if (cmdNameLookup) {
+         tmp = ExtractCommandLineFromAddressSpaceFile(&procInfo, &processInfo.procCmdName);
+      } else if (strlen(procInfo.pr_psargs) + 1 >= sizeof procInfo.pr_psargs) {
+         tmp = ExtractCommandLineFromAddressSpaceFile(&procInfo, NULL);
+      } else {
+         tmp = NULL;
+      }
+
+      if (tmp != NULL) {
+         processInfo.procCmdLine = Unicode_Alloc(tmp, STRING_ENCODING_DEFAULT);
+         cmdNameLookup = FALSE;
+      } else {
+         /*
+          * We had some issues reading procfs, mostly due to lack of
+          * permissions for certain system owned precesses. So let's resort to
+          * what the procinfo structure provides as a last resort.
+          */
+         processInfo.procCmdLine = Unicode_Alloc(procInfo.pr_psargs,
+                                                 STRING_ENCODING_DEFAULT);
+
+         if (cmdNameLookup) {
+            /*
+             * Now let's try and get the best effort command name from the entire
+             * command line. The method below does not take care of spaces in folder
+             * names and executable file names. This is the best we can do at this
+             * point, considering that spaces are not common in either file or
+             * folder names in Solaris, specially when owned by the system.
+             */
+            char *tmp2 = Unicode_Alloc(procInfo.pr_psargs, STRING_ENCODING_DEFAULT);
+            cmdNameBegin = tmp2;
+            /*
+             * Assuming the command name to end at the first blank space.
+             */
+            cmdNameEnd = cmdNameBegin;
+            while ('\0' != *cmdNameEnd) {
+               if ('/' == *cmdNameEnd) {
+                  cmdNameBegin = cmdNameEnd + 1;
+               }
+               if (' ' == *cmdNameEnd) {
+                  break;
+               }
+               cmdNameEnd++;
+            }
+            *cmdNameEnd = '\0';
+            processInfo.procCmdName = Str_SafeAsprintf(NULL, "%s", cmdNameBegin);
+            free(tmp2);
+         }
+      }
+      free(tmp);
+      tmp = NULL;
 
       /*
-       * Store the pid in dynbuf.
+       * Store the pid.
        */
-      pid = procInfo.pr_pid;
-      DynBuf_Append(&dbProcId, &pid, sizeof pid);
+      processInfo.procId = procInfo.pr_pid;
 
       /*
        * Store the owner of the process.
        */
       pwd = getpwuid(procInfo.pr_uid);
-      userName = (NULL == pwd)
-                 ? Str_Asprintf(&strLen, "%d", (int) procInfo.pr_uid)
-                 : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
-      DynBuf_Append(&dbProcOwner, &userName, sizeof userName);
+      processInfo.procOwner = (NULL == pwd)
+                              ? Str_SafeAsprintf(&strLen, "%d", (int) procInfo.pr_uid)
+                              : Unicode_Alloc(pwd->pw_name, STRING_ENCODING_DEFAULT);
 
       /*
-       * Store the time that the process started.
+       * Store the process info into a list buffer.
        */
-      DynBuf_Append(&dbProcStartTime,
-                    &processStartTime,
-                    sizeof processStartTime);
+      if (!ProcMgrProcInfoArray_Push(procList, processInfo)) {
+         Warning("%s: failed to expand DynArray - out of memory\n",
+                 __FUNCTION__);
+         goto exit;
+      }
+      processInfo.procCmdName = NULL;
+      processInfo.procCmdLine = NULL;
+      processInfo.procOwner = NULL;
    } // while (TRUE)
 
-   closedir(dir);
-
-   if (0 == DynBuf_GetSize(&dbProcId)) {
-      failed = TRUE;
-      goto exit;
+   if (0 < ProcMgrProcInfoArray_Count(procList)) {
+      failed = FALSE;
    }
 
-   /*
-    * We're done adding to DynBuf.  Trim off any unused allocated space.
-    * DynBuf_Trim() followed by DynBuf_Detach() avoids a memcpy().
-    */
-   DynBuf_Trim(&dbProcId);
-   DynBuf_Trim(&dbProcCmd);
-   DynBuf_Trim(&dbProcStartTime);
-   DynBuf_Trim(&dbProcOwner);
-
-   /*
-    * Create a ProcMgr_ProcList and populate its fields.
-    */
-   procList = (ProcMgr_ProcList *) Util_SafeCalloc(1, sizeof(ProcMgr_ProcList));
-   ASSERT_MEM_ALLOC(procList);
-
-   procList->procCount = DynBuf_GetSize(&dbProcId) / sizeof(pid_t);
-
-   procList->procIdList = (pid_t *) DynBuf_Detach(&dbProcId);
-   ASSERT_MEM_ALLOC(procList->procIdList);
-   procList->procCmdList = (char **) DynBuf_Detach(&dbProcCmd);
-   ASSERT_MEM_ALLOC(procList->procCmdList);
-   procList->startTime = (time_t *) DynBuf_Detach(&dbProcStartTime);
-   ASSERT_MEM_ALLOC(procList->startTime);
-   procList->procOwnerList = (char **) DynBuf_Detach(&dbProcOwner);
-   ASSERT_MEM_ALLOC(procList->procOwnerList);
-
 exit:
-   DynBuf_Destroy(&dbProcId);
-   DynBuf_Destroy(&dbProcCmd);
-   DynBuf_Destroy(&dbProcStartTime);
-   DynBuf_Destroy(&dbProcOwner);
+   closedir(dir);
+
+   free(processInfo.procOwner);
+   free(processInfo.procCmdLine);
+   free(processInfo.procCmdName);
 
    if (failed) {
       ProcMgr_FreeProcList(procList);
@@ -287,7 +301,8 @@ exit:
  */
 
 static char *
-ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo) //IN: psinfo struct
+ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo, //IN: psinfo struct
+                                       char **procCmdName) //OUT: command name
 {
    int argc;
    int i;
@@ -298,7 +313,11 @@ ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo) //IN: psinfo struct
    DynBuf cmdLine;
    DynBufArray args;
    pid_t pid;
+   char *cmdNameBegin;
 
+   if (NULL != procCmdName) {
+      *procCmdName = NULL;
+   }
    FileIO_Invalidate(&asFd);
    pid = procInfo->pr_pid;
 
@@ -331,6 +350,23 @@ ExtractCommandLineFromAddressSpaceFile(psinfo_t *procInfo) //IN: psinfo struct
          if (i + 1 < argc) {
             DynBuf_Append(&cmdLine, " ", 1);
          }
+         if (NULL != procCmdName && 0 == i) {
+            /*
+             * Store the command name of the process.
+             * Find the last path separator, to get the cmd name.
+             * If no separator is found, then use the whole name.
+             */
+            cmdNameBegin = strrchr(buf, '/');
+            if (NULL == cmdNameBegin) {
+               cmdNameBegin = buf;
+            } else {
+               /*
+                * Skip over the last separator.
+                */
+               cmdNameBegin++;
+            }
+            *procCmdName = Unicode_Alloc(cmdNameBegin, STRING_ENCODING_DEFAULT);
+         }
          DynBuf_Destroy(DynBufArray_AddressOf(&args, i));
       }
       DynBuf_AppendString(&cmdLine,"");
@@ -608,6 +644,7 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
    struct passwd *ppw;
    gid_t root_gid;
    int ret;
+   char *userLocal;
 
    ppw = &pw;
    if ((ppw = getpwuid_r(0, &pw, buffer, sizeof buffer)) == NULL) {
@@ -616,7 +653,18 @@ ProcMgr_ImpersonateUserStart(const char *user,  // IN: UTF-8 encoded user name
 
    root_gid = ppw->pw_gid;
 
-   if ((ppw = getpwnam_r(user, &pw, buffer, sizeof buffer)) == NULL) {
+   /* convert user name to local character set */
+   userLocal = (char *)Unicode_GetAllocBytes(user, Unicode_GetCurrentEncoding());
+   if (!userLocal) {
+       Warning("Failed to convert user name %s to local character set.\n", user);
+       return FALSE;
+   }
+
+   ppw = getpwnam_r(userLocal, &pw, buffer, sizeof buffer);
+
+   free(userLocal);
+
+   if (ppw == NULL) {
       return FALSE;
    }
 
