@@ -254,26 +254,6 @@ VmCheck_IsVirtualWorld(void)
    uint32 dummy;
 
 #if !defined(WINNT_DDK)
-   char *hypervisorSig;
-
-   hypervisorSig = Hostinfo_HypervisorCPUIDSig();
-   if (hypervisorSig != NULL &&
-       *hypervisorSig != '\0' &&
-       strcmp(hypervisorSig, CPUID_VMWARE_HYPERVISOR_VENDOR_STRING) != 0) {
-      free(hypervisorSig);
-      Debug("%s: detected non-VMware hypervisor (%s).\n",
-            __FUNCTION__, hypervisorSig);
-      return FALSE;
-   }
-
-   free(hypervisorSig);
-
-   /*
-    * No hypervisor CPUID string, check through other means, trying the
-    * backdoor as a last resort since some hypervisors don't play well
-    * with it.
-    */
-
    if (VmCheckSafe(Hostinfo_TouchXen)) {
       Debug("%s: detected Xen.\n", __FUNCTION__);
       return FALSE;
