@@ -22,8 +22,8 @@
 #define INCLUDE_ALLOW_VMMON
 #define INCLUDE_ALLOW_MODULE
 #define INCLUDE_ALLOW_VMCORE
-#define INCLUDE_ALLOW_VMNIXMOD
 #define INCLUDE_ALLOW_DISTRIBUTE
+#define INCLUDE_ALLOW_VMKDRIVERS
 #include "includeCheck.h"
 
 
