@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2010 VMware, Inc. All rights reserved.
+ * Copyright (C) 2010-2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -294,7 +294,14 @@ VMCIQPairMapQueueHeaders(VMCIQueue *produceQ, // IN
 
    if (NULL == produceQ->qHeader || NULL == consumeQ->qHeader) {
       if (canBlock) {
-         result = VMCIHost_MapQueues(produceQ, consumeQ, 0);
+         /*
+          * We return data from creator of the queue in VM2VM case.
+          * That should be OK, as if they do not match then there
+          * is somebody else in progress of making them match, and
+          * you should not be looking at somebody else's queue if
+          * queue is active.
+          */
+         result = VMCIHost_MapQueues(0, produceQ, consumeQ, 0);
       } else {
          result = VMCI_ERROR_QUEUEPAIR_NOT_READY;
       }
@@ -411,7 +418,7 @@ VMCIQPairWaitForReadyQueue(VMCIQPair *qpair)
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_Alloc --
+ * vmci_qpair_alloc --
  *
  *      This is the client interface for allocating the memory for a
  *      VMCIQPair structure and then attaching to the underlying
@@ -429,15 +436,15 @@ VMCIQPairWaitForReadyQueue(VMCIQPair *qpair)
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_Alloc)
+VMCI_EXPORT_SYMBOL(vmci_qpair_alloc)
 int
-VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
-                VMCIHandle *handle,           // OUT
-                uint64 produceQSize,          // IN
-                uint64 consumeQSize,          // IN
-                VMCIId peer,                  // IN
-                uint32 flags,                 // IN
-                VMCIPrivilegeFlags privFlags) // IN
+vmci_qpair_alloc(VMCIQPair **qpair,            // OUT
+                 VMCIHandle *handle,           // OUT
+                 uint64 produceQSize,          // IN
+                 uint64 consumeQSize,          // IN
+                 VMCIId peer,                  // IN
+                 uint32 flags,                 // IN
+                 VMCIPrivilegeFlags privFlags) // IN
 {
    VMCIQPair *myQPair;
    int retval;
@@ -509,7 +516,6 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
    if (!myQPair) {
       return VMCI_ERROR_NO_MEM;
    }
-   memset(myQPair, 0, sizeof *myQPair);
 
    myQPair->produceQSize = produceQSize;
    myQPair->consumeQSize = consumeQSize;
@@ -560,7 +566,7 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_Detach --
+ * vmci_qpair_detach --
  *
  *      This is the client interface for detaching from a VMCIQPair.
  *      Note that this routine will free the memory allocated for the
@@ -575,9 +581,9 @@ VMCIQPair_Alloc(VMCIQPair **qpair,            // OUT
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_Detach)
+VMCI_EXPORT_SYMBOL(vmci_qpair_detach)
 int
-VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
+vmci_qpair_detach(VMCIQPair **qpair) // IN/OUT
 {
    int result;
    VMCIQPair *oldQPair;
@@ -600,9 +606,6 @@ VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
    if (!(oldQPair->guestEndpoint || (oldQPair->flags & VMCI_QPFLAG_LOCAL))) {
       VMCI_DestroyEvent(&oldQPair->event);
    }
-   memset(oldQPair, 0, sizeof *oldQPair);
-   oldQPair->handle = VMCI_INVALID_HANDLE;
-   oldQPair->peer = VMCI_INVALID_ID;
    VMCI_FreeKernelMem(oldQPair, sizeof *oldQPair);
    *qpair = NULL;
 
@@ -613,7 +616,7 @@ VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_GetProduceIndexes --
+ * vmci_qpair_get_produce_indexes --
  *
  *      This is the client interface for getting the current indexes of the
  *      QPair from the point of the view of the caller as the producer.
@@ -628,11 +631,11 @@ VMCIQPair_Detach(VMCIQPair **qpair) // IN/OUT
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_GetProduceIndexes)
+VMCI_EXPORT_SYMBOL(vmci_qpair_get_produce_indexes)
 int
-VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
-                            uint64 *producerTail,   // OUT
-                            uint64 *consumerHead)   // OUT
+vmci_qpair_get_produce_indexes(const VMCIQPair *qpair, // IN
+                               uint64 *producerTail,   // OUT
+                               uint64 *consumerHead)   // OUT
 {
    VMCIQueueHeader *produceQHeader;
    VMCIQueueHeader *consumeQHeader;
@@ -663,7 +666,7 @@ VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_GetConsumeIndexes --
+ * vmci_qpair_get_consume_indexes --
  *
  *      This is the client interface for getting the current indexes of the
  *      QPair from the point of the view of the caller as the consumer.
@@ -678,11 +681,11 @@ VMCIQPair_GetProduceIndexes(const VMCIQPair *qpair, // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_GetConsumeIndexes)
+VMCI_EXPORT_SYMBOL(vmci_qpair_get_consume_indexes)
 int
-VMCIQPair_GetConsumeIndexes(const VMCIQPair *qpair, // IN
-                            uint64 *consumerTail,   // OUT
-                            uint64 *producerHead)   // OUT
+vmci_qpair_get_consume_indexes(const VMCIQPair *qpair, // IN
+                               uint64 *consumerTail,   // OUT
+                               uint64 *producerHead)   // OUT
 {
    VMCIQueueHeader *produceQHeader;
    VMCIQueueHeader *consumeQHeader;
@@ -713,7 +716,7 @@ VMCIQPair_GetConsumeIndexes(const VMCIQPair *qpair, // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_ProduceFreeSpace --
+ * vmci_qpair_produce_free_space --
  *
  *      This is the client interface for getting the amount of free
  *      space in the QPair from the point of the view of the caller as
@@ -730,9 +733,9 @@ VMCIQPair_GetConsumeIndexes(const VMCIQPair *qpair, // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_ProduceFreeSpace)
+VMCI_EXPORT_SYMBOL(vmci_qpair_produce_free_space)
 int64
-VMCIQPair_ProduceFreeSpace(const VMCIQPair *qpair) // IN
+vmci_qpair_produce_free_space(const VMCIQPair *qpair) // IN
 {
    VMCIQueueHeader *produceQHeader;
    VMCIQueueHeader *consumeQHeader;
@@ -759,7 +762,7 @@ VMCIQPair_ProduceFreeSpace(const VMCIQPair *qpair) // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_ConsumeFreeSpace --
+ * vmci_qpair_consume_free_space --
  *
  *      This is the client interface for getting the amount of free
  *      space in the QPair from the point of the view of the caller as
@@ -777,9 +780,9 @@ VMCIQPair_ProduceFreeSpace(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_ConsumeFreeSpace)
+VMCI_EXPORT_SYMBOL(vmci_qpair_consume_free_space)
 int64
-VMCIQPair_ConsumeFreeSpace(const VMCIQPair *qpair) // IN
+vmci_qpair_consume_free_space(const VMCIQPair *qpair) // IN
 {
    VMCIQueueHeader *produceQHeader;
    VMCIQueueHeader *consumeQHeader;
@@ -806,7 +809,7 @@ VMCIQPair_ConsumeFreeSpace(const VMCIQPair *qpair) // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_ProduceBufReady --
+ * vmci_qpair_produce_buf_ready --
  *
  *      This is the client interface for getting the amount of
  *      enqueued data in the QPair from the point of the view of the
@@ -824,9 +827,9 @@ VMCIQPair_ConsumeFreeSpace(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_ProduceBufReady)
+VMCI_EXPORT_SYMBOL(vmci_qpair_produce_buf_ready)
 int64
-VMCIQPair_ProduceBufReady(const VMCIQPair *qpair) // IN
+vmci_qpair_produce_buf_ready(const VMCIQPair *qpair) // IN
 {
    VMCIQueueHeader *produceQHeader;
    VMCIQueueHeader *consumeQHeader;
@@ -853,7 +856,7 @@ VMCIQPair_ProduceBufReady(const VMCIQPair *qpair) // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_ConsumeBufReady --
+ * vmci_qpair_consume_buf_ready --
  *
  *      This is the client interface for getting the amount of
  *      enqueued data in the QPair from the point of the view of the
@@ -870,9 +873,9 @@ VMCIQPair_ProduceBufReady(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_ConsumeBufReady)
+VMCI_EXPORT_SYMBOL(vmci_qpair_consume_buf_ready)
 int64
-VMCIQPair_ConsumeBufReady(const VMCIQPair *qpair) // IN
+vmci_qpair_consume_buf_ready(const VMCIQPair *qpair) // IN
 {
    VMCIQueueHeader *produceQHeader;
    VMCIQueueHeader *consumeQHeader;
@@ -922,7 +925,7 @@ VMCIQPair_ConsumeBufReady(const VMCIQPair *qpair) // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE ssize_t
+static ssize_t
 EnqueueLocked(VMCIQueue *produceQ,                   // IN
               VMCIQueue *consumeQ,                   // IN
               const uint64 produceQSize,             // IN
@@ -1008,7 +1011,7 @@ EnqueueLocked(VMCIQueue *produceQ,                   // IN
  *-----------------------------------------------------------------------------
  */
 
-static INLINE ssize_t
+static ssize_t
 DequeueLocked(VMCIQueue *produceQ,                        // IN
               VMCIQueue *consumeQ,                        // IN
               const uint64 consumeQSize,                  // IN
@@ -1074,7 +1077,7 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_Enqueue --
+ * vmci_qpair_enqueue --
  *
  *      This is the client interface for enqueueing data into the queue.
  *
@@ -1088,12 +1091,12 @@ DequeueLocked(VMCIQueue *produceQ,                        // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_Enqueue)
+VMCI_EXPORT_SYMBOL(vmci_qpair_enqueue)
 ssize_t
-VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
-                  const void *buf,         // IN
-                  size_t bufSize,          // IN
-                  int bufType)             // IN
+vmci_qpair_enqueue(VMCIQPair *qpair,        // IN
+                   const void *buf,         // IN
+                   size_t bufSize,          // IN
+                   int bufType)             // IN
 {
    ssize_t result;
 
@@ -1131,7 +1134,7 @@ VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_Dequeue --
+ * vmci_qpair_dequeue --
  *
  *      This is the client interface for dequeueing data from the queue.
  *
@@ -1145,12 +1148,12 @@ VMCIQPair_Enqueue(VMCIQPair *qpair,        // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_Dequeue)
+VMCI_EXPORT_SYMBOL(vmci_qpair_dequeue)
 ssize_t
-VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
-                  void *buf,               // IN
-                  size_t bufSize,          // IN
-                  int bufType)             // IN
+vmci_qpair_dequeue(VMCIQPair *qpair,        // IN
+                   void *buf,               // IN
+                   size_t bufSize,          // IN
+                   int bufType)             // IN
 {
    ssize_t result;
 
@@ -1188,7 +1191,7 @@ VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_Peek --
+ * vmci_qpair_peek --
  *
  *      This is the client interface for peeking into a queue.  (I.e.,
  *      copy data from the queue without updating the head pointer.)
@@ -1203,12 +1206,12 @@ VMCIQPair_Dequeue(VMCIQPair *qpair,        // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_Peek)
+VMCI_EXPORT_SYMBOL(vmci_qpair_peek)
 ssize_t
-VMCIQPair_Peek(VMCIQPair *qpair,    // IN
-               void *buf,           // IN
-               size_t bufSize,      // IN
-               int bufType)         // IN
+vmci_qpair_peek(VMCIQPair *qpair,    // IN
+                void *buf,           // IN
+                size_t bufSize,      // IN
+                int bufType)         // IN
 {
    ssize_t result;
 
@@ -1250,7 +1253,7 @@ VMCIQPair_Peek(VMCIQPair *qpair,    // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_EnqueueV --
+ * vmci_qpair_enquev --
  *
  *      This is the client interface for enqueueing data into the queue.
  *
@@ -1264,12 +1267,12 @@ VMCIQPair_Peek(VMCIQPair *qpair,    // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_EnqueueV)
+VMCI_EXPORT_SYMBOL(vmci_qpair_enquev)
 ssize_t
-VMCIQPair_EnqueueV(VMCIQPair *qpair,        // IN
-                   void *iov,               // IN
-                   size_t iovSize,          // IN
-                   int bufType)             // IN
+vmci_qpair_enquev(VMCIQPair *qpair,        // IN
+                  void *iov,               // IN
+                  size_t iovSize,          // IN
+                  int bufType)             // IN
 {
    ssize_t result;
 
@@ -1307,7 +1310,7 @@ VMCIQPair_EnqueueV(VMCIQPair *qpair,        // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_DequeueV --
+ * vmci_qpair_dequev --
  *
  *      This is the client interface for dequeueing data from the queue.
  *
@@ -1321,24 +1324,24 @@ VMCIQPair_EnqueueV(VMCIQPair *qpair,        // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_DequeueV)
+VMCI_EXPORT_SYMBOL(vmci_qpair_dequev)
 ssize_t
-VMCIQPair_DequeueV(VMCIQPair *qpair,         // IN
-                   void *iov,                // IN
-                   size_t iovSize,           // IN
-                   int bufType)              // IN
+vmci_qpair_dequev(VMCIQPair *qpair,         // IN
+                  void *iov,                // IN
+                  size_t iovSize,           // IN
+                  int bufType)              // IN
 {
    ssize_t result;
 
+   if (!qpair || !iov) {
+      return VMCI_ERROR_INVALID_ARGS;
+   }
+
    result = VMCIQPairLock(qpair);
    if (result != VMCI_SUCCESS) {
       return result;
    }
 
-   if (!qpair || !iov) {
-      return VMCI_ERROR_INVALID_ARGS;
-   }
-
    do {
       result = DequeueLocked(qpair->produceQ,
                              qpair->consumeQ,
@@ -1364,7 +1367,7 @@ VMCIQPair_DequeueV(VMCIQPair *qpair,         // IN
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIQPair_PeekV --
+ * vmci_qpair_peekv --
  *
  *      This is the client interface for peeking into a queue.  (I.e.,
  *      copy data from the queue without updating the head pointer.)
@@ -1379,12 +1382,12 @@ VMCIQPair_DequeueV(VMCIQPair *qpair,         // IN
  *-----------------------------------------------------------------------------
  */
 
-VMCI_EXPORT_SYMBOL(VMCIQPair_PeekV)
+VMCI_EXPORT_SYMBOL(vmci_qpair_peekv)
 ssize_t
-VMCIQPair_PeekV(VMCIQPair *qpair,           // IN
-                void *iov,                  // IN
-                size_t iovSize,             // IN
-                int bufType)                // IN
+vmci_qpair_peekv(VMCIQPair *qpair,           // IN
+                 void *iov,                  // IN
+                 size_t iovSize,             // IN
+                 int bufType)                // IN
 {
    ssize_t result;
 
