@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2011 VMware, Inc. All rights reserved.
+ * Copyright (C) 2011-2012 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -114,6 +114,17 @@ VMCI_Route(VMCIHandle *src,       // IN/OUT
          return VMCI_ERROR_INVALID_ARGS;
       }
 
+      /*
+       * If the client passed the ANON source handle then respect it (both
+       * context and resource are invalid).  However, if they passed only
+       * an invalid context, then they probably mean ANY, in which case we
+       * should set the real context here before passing it down.
+       */
+
+      if (VMCI_INVALID_ID == src->context && VMCI_INVALID_ID != src->resource) {
+         src->context = vmci_get_context_id();
+      }
+
       /* Send from local client down to the hypervisor. */
       *route = VMCI_ROUTE_AS_GUEST;
       return VMCI_SUCCESS;
@@ -148,7 +159,7 @@ VMCI_Route(VMCIHandle *src,       // IN/OUT
       if (!fromGuest && hasGuestDevice) {
          /* If no source context then use the current. */
          if (VMCI_INVALID_ID == src->context) {
-            src->context = VMCI_GetContextID();
+            src->context = vmci_get_context_id();
          }
 
          /* Send it from local client down to the host. */
@@ -214,6 +225,13 @@ VMCI_Route(VMCIHandle *src,       // IN/OUT
          /* Pass it up to the guest. */
          *route = VMCI_ROUTE_AS_HOST;
          return VMCI_SUCCESS;
+      } else if (!hasGuestDevice) {
+         /*
+          * The host is attempting to reach a CID without an active context, and
+          * we can't send it down, since we have no guest device.
+          */
+
+         return VMCI_ERROR_DST_UNREACHABLE;
       }
    }
 
@@ -225,12 +243,20 @@ VMCI_Route(VMCIHandle *src,       // IN/OUT
     */
 
    if (!hasGuestDevice) {
+      /*
+       * Ending up here means we have neither guest nor host device. That
+       * shouldn't happen, since any VMCI client in the kernel should have done
+       * a successful VMCI_DeviceGet.
+       */
+
+      ASSERT(FALSE);
+
       return VMCI_ERROR_DEVICE_NOT_FOUND;
    }
 
    /* If no source context then use the current context. */
    if (VMCI_INVALID_ID == src->context) {
-      src->context = VMCI_GetContextID();
+      src->context = vmci_get_context_id();
    }
 
    /*
