@@ -69,9 +69,11 @@
 struct VMCIQueueKernelIf {
    struct page **page;
    struct page **headerPage;
+   void *va;
    VMCIMutex __mutex;
    VMCIMutex *mutex;
    Bool host;
+   Bool isDataMapped;
    size_t numPages;
 };
 
@@ -885,7 +887,8 @@ VMCIMutex_Release(VMCIMutex *mutex) // IN:
  */
 
 void *
-VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
+VMCI_AllocQueue(uint64 size,  // IN: size of queue (not including header)
+                uint32 flags) // IN: queuepair flags
 {
    uint64 i;
    VMCIQueue *queue;
@@ -902,11 +905,21 @@ VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
     * unresponsive, because we allocate page-by-page, and we allow the
     * system to wait for pages rather than fail.
     */
+
    if (size > VMCI_MAX_GUEST_QP_MEMORY) {
       ASSERT(FALSE);
       return NULL;
    }
 
+   /*
+    * If pinning is requested then double-check the size of the queue.
+    * VMCIQPair_Alloc() will do this for the total queuepair size.
+    */
+
+   if ((flags & VMCI_QPFLAG_PINNED) && size > VMCI_MAX_PINNED_QP_MEMORY) {
+      return NULL;
+   }
+
    qHeader = (VMCIQueueHeader *)vmalloc(queueSize);
    if (!qHeader) {
       return NULL;
@@ -919,19 +932,35 @@ VMCI_AllocQueue(uint64 size) // IN: size of queue (not including header)
    queue->kernelIf->headerPage = NULL; // Unused in guest.
    queue->kernelIf->page = (struct page **)((uint8 *)queue->kernelIf +
                                             sizeof *(queue->kernelIf));
+   queue->kernelIf->va = NULL;
    queue->kernelIf->host = FALSE;
+   queue->kernelIf->isDataMapped = FALSE;
 
    for (i = 0; i < numDataPages; i++) {
       queue->kernelIf->page[i] = alloc_pages(GFP_KERNEL, 0);
       if (!queue->kernelIf->page[i]) {
-         while (i) {
-            __free_page(queue->kernelIf->page[--i]);
-         }
-         vfree(qHeader);
+         VMCI_FreeQueue(queue, i * PAGE_SIZE);
          return NULL;
       }
    }
 
+   /*
+    * alloc_pages() returns pinned PAs, but we need a permanent mapping to VA
+    * if the caller has requested pinned queuepairs.  Map all of them into
+    * kernel VA now, for the lifetime of the queue.  The page VAs will be
+    * contiguous.
+    */
+
+   if (flags & VMCI_QPFLAG_PINNED) {
+      queue->kernelIf->va = vmap(queue->kernelIf->page, numDataPages, VM_MAP,
+                                 PAGE_KERNEL);
+      if (NULL == queue->kernelIf->va) {
+         VMCI_FreeQueue(queue, numDataPages * PAGE_SIZE);
+         return NULL;
+      }
+      queue->kernelIf->isDataMapped = TRUE;
+   }
+
    return (void *)queue;
 }
 
@@ -961,6 +990,15 @@ VMCI_FreeQueue(void *q,     // IN:
 
    if (queue) {
       uint64 i;
+
+      if (queue->kernelIf->isDataMapped) {
+         ASSERT(queue->kernelIf->va);
+         vunmap(queue->kernelIf->va);
+         queue->kernelIf->va = NULL;
+      }
+
+      ASSERT(NULL == queue->kernelIf->va);
+
       for (i = 0; i < CEILING(size, PAGE_SIZE); i++) {
          __free_page(queue->kernelIf->page[i]);
       }
@@ -1169,9 +1207,15 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
    while (bytesCopied < size) {
       uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
       size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
-      void *va = kmap(kernelIf->page[pageIndex]);
+      void *va;
       size_t toCopy;
 
+      if (kernelIf->isDataMapped) {
+         va = (void *)((uint8 *)kernelIf->va + (pageIndex * PAGE_SIZE));
+      } else {
+         va = kmap(kernelIf->page[pageIndex]);
+      }
+
       ASSERT(va);
       if (size - bytesCopied > PAGE_SIZE - pageOffset) {
          /* Enough payload to fill up from this page. */
@@ -1187,7 +1231,9 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_fromiovec((uint8 *)va + pageOffset, iov, toCopy);
          if (err != 0) {
-            kunmap(kernelIf->page[pageIndex]);
+            if (!kernelIf->isDataMapped) {
+               kunmap(kernelIf->page[pageIndex]);
+            }
             return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
@@ -1195,7 +1241,9 @@ __VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
       }
 
       bytesCopied += toCopy;
-      kunmap(kernelIf->page[pageIndex]);
+      if (!kernelIf->isDataMapped) {
+         kunmap(kernelIf->page[pageIndex]);
+      }
    }
 
    return VMCI_SUCCESS;
@@ -1234,9 +1282,15 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
    while (bytesCopied < size) {
       uint64 pageIndex = (queueOffset + bytesCopied) / PAGE_SIZE;
       size_t pageOffset = (queueOffset + bytesCopied) & (PAGE_SIZE - 1);
-      void *va = kmap(kernelIf->page[pageIndex]);
+      void *va;
       size_t toCopy;
 
+      if (kernelIf->isDataMapped) {
+         va = (void *)((uint8 *)kernelIf->va + (pageIndex * PAGE_SIZE));
+      } else {
+         va = kmap(kernelIf->page[pageIndex]);
+      }
+
       ASSERT(va);
       if (size - bytesCopied > PAGE_SIZE - pageOffset) {
          /* Enough payload to fill up this page. */
@@ -1252,7 +1306,9 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
          /* The iovec will track bytesCopied internally. */
          err = memcpy_toiovec(iov, (uint8 *)va + pageOffset, toCopy);
          if (err != 0) {
-            kunmap(kernelIf->page[pageIndex]);
+            if (!kernelIf->isDataMapped) {
+               kunmap(kernelIf->page[pageIndex]);
+            }
             return VMCI_ERROR_INVALID_ARGS;
          }
       } else {
@@ -1260,7 +1316,9 @@ __VMCIMemcpyFromQueue(void *dest,             // OUT:
       }
 
       bytesCopied += toCopy;
-      kunmap(kernelIf->page[pageIndex]);
+      if (!kernelIf->isDataMapped) {
+         kunmap(kernelIf->page[pageIndex]);
+      }
    }
 
    return VMCI_SUCCESS;
@@ -1292,7 +1350,7 @@ VMCIMemcpyToQueue(VMCIQueue *queue,   // OUT:
                   int bufType,        // IN: Unused
                   Bool canBlock)      // IN: Unused
 {
-   ASSERT(canBlock);
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    return __VMCIMemcpyToQueue(queue, queueOffset,
                               (uint8 *)src + srcOffset, size, FALSE);
@@ -1324,7 +1382,7 @@ VMCIMemcpyFromQueue(void *dest,             // OUT:
                     int bufType,            // IN: Unused
                     Bool canBlock)          // IN: Unused
 {
-   ASSERT(canBlock);
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    return __VMCIMemcpyFromQueue((uint8 *)dest + destOffset,
                                 queue, queueOffset, size, FALSE);
@@ -1357,7 +1415,7 @@ VMCIMemcpyToQueueLocal(VMCIQueue *queue,     // OUT
                        int bufType,          // IN: Unused
                        Bool canBlock)        // IN: Unused
 {
-   ASSERT(canBlock);
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    return __VMCIMemcpyToQueue(queue, queueOffset,
                               (uint8 *)src + srcOffset, size, FALSE);;
@@ -1389,7 +1447,7 @@ VMCIMemcpyFromQueueLocal(void *dest,             // OUT:
                          int bufType,            // IN: Unused
                          Bool canBlock)          // IN: Unused
 {
-   ASSERT(canBlock);
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    return __VMCIMemcpyFromQueue((uint8 *)dest + destOffset,
                                 queue, queueOffset, size, FALSE);
@@ -1421,7 +1479,7 @@ VMCIMemcpyToQueueV(VMCIQueue *queue,      // OUT:
                    int bufType,           // IN: Unused
                    Bool canBlock)         // IN: Unused
 {
-   ASSERT(canBlock);
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    /*
     * We ignore srcOffset because src is really a struct iovec * and will
@@ -1456,7 +1514,7 @@ VMCIMemcpyFromQueueV(void *dest,              // OUT: iovec
                      int bufType,             // IN: Unused
                      Bool canBlock)           // IN: Unused
 {
-   ASSERT(canBlock);
+   ASSERT(canBlock || !queue->kernelIf->host);
 
    /*
     * We ignore destOffset because dest is really a struct iovec * and will
@@ -1540,6 +1598,8 @@ VMCIHost_AllocQueue(uint64 size) // IN:
       queue->kernelIf->page = &queue->kernelIf->headerPage[1];
       memset(queue->kernelIf->headerPage, 0,
              sizeof *queue->kernelIf->headerPage * queue->kernelIf->numPages);
+      queue->kernelIf->va = NULL;
+      queue->kernelIf->isDataMapped = FALSE;
    }
 
    return queue;
@@ -1672,9 +1732,9 @@ VMCI_AcquireQueueMutex(VMCIQueue *queue, // IN
 {
    ASSERT(queue);
    ASSERT(queue->kernelIf);
-   ASSERT(canBlock);
 
    if (queue->kernelIf->host) {
+      ASSERT(canBlock);
       ASSERT(queue->kernelIf->mutex);
       down(queue->kernelIf->mutex);
    }
@@ -1714,6 +1774,64 @@ VMCI_ReleaseQueueMutex(VMCIQueue *queue) // IN
 }
 
 
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_LockQueueHeader()
+ *
+ *       Acquire a spinlock guarding the queue header.  Note that the produceQ
+ *       and the consumeQ share the lock mutex.  So, only one of the two need to
+ *       be passed in to this routine.  Either will work just fine.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_LockQueueHeader(VMCIQueue *queue) // IN
+{
+   ASSERT(queue);
+   ASSERT(queue->kernelIf);
+   ASSERT(!queue->kernelIf->host);
+
+   /*
+    * We don't support non-blocking on the host right now, so we won't get
+    * here for a host queue.  And there's no lock required on the guest.  So
+    * this is a NOP.
+    */
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * VMCI_UnlockQueueHeader()
+ *
+ *       Release the spinlock guarding the queue header.
+ *
+ * Results:
+ *       None.
+ *
+ * Side Effects:
+ *       None.
+ *
+ *----------------------------------------------------------------------------
+ */
+
+void
+VMCI_UnlockQueueHeader(VMCIQueue *queue) // IN
+{
+   ASSERT(queue);
+   ASSERT(queue->kernelIf);
+   ASSERT(!queue->kernelIf->host);
+}
+
+
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1768,13 +1886,15 @@ VMCIReleasePages(struct page **pages,  // IN
  */
 
 int
-VMCIHost_RegisterUserMemory(QueuePairPageStore *pageStore,  // IN
+VMCIHost_RegisterUserMemory(unsigned int index,             // IN
+                            QueuePairPageStore *pageStore,  // IN
                             VMCIQueue *produceQ,            // OUT
                             VMCIQueue *consumeQ)            // OUT
 {
    VA64 produceUVA;
    VA64 consumeUVA;
 
+   ASSERT(index == 0);
    ASSERT(produceQ->kernelIf->headerPage && consumeQ->kernelIf->headerPage);
 
    /*
@@ -1785,7 +1905,7 @@ VMCIHost_RegisterUserMemory(QueuePairPageStore *pageStore,  // IN
 
    produceUVA = pageStore->pages;
    consumeUVA = pageStore->pages + produceQ->kernelIf->numPages * PAGE_SIZE;
-   return VMCIHost_GetUserMemory(produceUVA, consumeUVA, produceQ, consumeQ);
+   return VMCIHost_GetUserMemory(index, produceUVA, consumeUVA, produceQ, consumeQ);
 }
 
 
@@ -1808,9 +1928,11 @@ VMCIHost_RegisterUserMemory(QueuePairPageStore *pageStore,  // IN
  */
 
 void
-VMCIHost_UnregisterUserMemory(VMCIQueue *produceQ,         // IN/OUT
+VMCIHost_UnregisterUserMemory(unsigned int index,          // IN
+                              VMCIQueue *produceQ,         // IN/OUT
                               VMCIQueue *consumeQ)         // IN/OUT
 {
+   ASSERT(index == 0);
    ASSERT(produceQ->kernelIf);
    ASSERT(consumeQ->kernelIf);
    ASSERT(!produceQ->qHeader && !consumeQ->qHeader);
@@ -1827,12 +1949,12 @@ VMCIHost_UnregisterUserMemory(VMCIQueue *produceQ,         // IN/OUT
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIHost_MapQueueHeaders --
+ * VMCIHost_MapQueues --
  *
  *       Once VMCIHost_RegisterUserMemory has been performed on a
  *       queue, the queue pair headers can be mapped into the
  *       kernel. Once mapped, they must be unmapped with
- *       VMCIHost_UnmapQueueHeaders prior to calling
+ *       VMCIHost_UnmapQueues prior to calling
  *       VMCIHost_UnregisterUserMemory.
  *
  * Results:
@@ -1845,11 +1967,14 @@ VMCIHost_UnregisterUserMemory(VMCIQueue *produceQ,         // IN/OUT
  */
 
 int
-VMCIHost_MapQueueHeaders(VMCIQueue *produceQ,  // IN/OUT
-                         VMCIQueue *consumeQ)  // IN/OUT
+VMCIHost_MapQueues(unsigned int index,   // IN
+                   VMCIQueue *produceQ,  // IN/OUT
+                   VMCIQueue *consumeQ,  // IN/OUT
+                   uint32 flags)         // UNUSED
 {
    int result;
 
+   ASSERT(index == 0);
    if (!produceQ->qHeader || !consumeQ->qHeader) {
       struct page *headers[2];
 
@@ -1887,7 +2012,7 @@ VMCIHost_MapQueueHeaders(VMCIQueue *produceQ,  // IN/OUT
 /*
  *-----------------------------------------------------------------------------
  *
- * VMCIHost_UnmapQueueHeaders --
+ * VMCIHost_UnmapQueues --
  *
  *       Unmaps previously mapped queue pair headers from the kernel.
  *
@@ -1901,10 +2026,12 @@ VMCIHost_MapQueueHeaders(VMCIQueue *produceQ,  // IN/OUT
  */
 
 int
-VMCIHost_UnmapQueueHeaders(VMCIGuestMemID gid,   // IN
-                           VMCIQueue *produceQ,  // IN/OUT
-                           VMCIQueue *consumeQ)  // IN/OUT
+VMCIHost_UnmapQueues(unsigned int index,   // IN
+                     VMCIGuestMemID gid,   // IN
+                     VMCIQueue *produceQ,  // IN/OUT
+                     VMCIQueue *consumeQ)  // IN/OUT
 {
+   ASSERT(index == 0);
    if (produceQ->qHeader) {
       ASSERT(consumeQ->qHeader);
 
@@ -1941,7 +2068,8 @@ VMCIHost_UnmapQueueHeaders(VMCIGuestMemID gid,   // IN
  */
 
 int
-VMCIHost_GetUserMemory(VA64 produceUVA,       // IN
+VMCIHost_GetUserMemory(unsigned int index,    // IN
+                       VA64 produceUVA,       // IN
                        VA64 consumeUVA,       // IN
                        VMCIQueue *produceQ,   // OUT
                        VMCIQueue *consumeQ)   // OUT
@@ -1949,6 +2077,7 @@ VMCIHost_GetUserMemory(VA64 produceUVA,       // IN
    int retval;
    int err = VMCI_SUCCESS;
 
+   ASSERT(index == 0);
    down_write(&current->mm->mmap_sem);
    retval = get_user_pages(current,
                            current->mm,
@@ -2004,12 +2133,14 @@ out:
  */
 
 void
-VMCIHost_ReleaseUserMemory(VMCIQueue *produceQ,  // IN/OUT
+VMCIHost_ReleaseUserMemory(unsigned int index,   // IN
+                           VMCIQueue *produceQ,  // IN/OUT
                            VMCIQueue *consumeQ)  // IN/OUT
 {
+   ASSERT(index == 0);
    ASSERT(produceQ->kernelIf->headerPage);
 
-   VMCIHost_UnregisterUserMemory(produceQ, consumeQ);
+   VMCIHost_UnregisterUserMemory(index, produceQ, consumeQ);
 }
 
 
