@@ -33,6 +33,11 @@
 #include "compat_kernel.h"
 #include "compat_slab.h"
 
+/* Must be after compat_fs.h */
+#if defined VMW_USE_AIO
+#include <linux/aio.h>
+#endif
+
 #include "cpName.h"
 #include "hgfsProto.h"
 #include "module.h"
@@ -83,6 +88,9 @@ static loff_t HgfsSeek(struct file *file,
 static int HgfsFsync(struct file *file,
 #if defined VMW_FSYNC_OLD
                      struct dentry *dentry,
+#elif defined VMW_FSYNC_31
+                     loff_t start,
+                     loff_t end,
 #endif
                      int datasync);
 static int HgfsMmap(struct file *file,
@@ -989,6 +997,9 @@ static int
 HgfsFsync(struct file *file,		// IN: File we operate on
 #if defined VMW_FSYNC_OLD
           struct dentry *dentry,        // IN: Dentry for this file
+#elif defined VMW_FSYNC_31
+          loff_t start,                 // IN: start of range to sync
+          loff_t end,                   // IN: end of range to sync
 #endif
           int datasync)	                // IN: fdatasync or fsync
 {
