@@ -30,10 +30,10 @@
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
 #include <linux/namei.h>
 #endif
+#include <linux/highmem.h>
 
 #include "compat_cred.h"
 #include "compat_fs.h"
-#include "compat_highmem.h"
 #include "compat_kernel.h"
 #include "compat_mm.h"
 #include "compat_page-flags.h"
@@ -75,10 +75,20 @@ static int HgfsPackSymlinkCreateRequest(struct dentry *dentry,
 static int HgfsCreate(struct inode *dir,
                       struct dentry *dentry,
                       compat_umode_t mode,
-                      struct nameidata *nd);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
+                      bool excl
+#else
+                      struct nameidata *nd
+#endif
+);
 static struct dentry *HgfsLookup(struct inode *dir,
                                  struct dentry *dentry,
-                                 struct nameidata *nd);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
+                                 unsigned int flags
+#else
+                                 struct nameidata *nd
+#endif
+);
 static int HgfsMkdir(struct inode *dir,
                      struct dentry *dentry,
                      compat_umode_t mode);
@@ -97,7 +107,7 @@ static int HgfsSymlink(struct inode *dir,
 static int HgfsPermission(struct inode *inode,
                           int mask,
                           struct nameidata *nameidata);
-#elif defined(IPERM_FLAG_RCU) /* introduced in 2.6.38 */
+#elif defined(IPERM_FLAG_RCU)
 static int HgfsPermission(struct inode *inode,
                           int mask,
                           unsigned int flags);
@@ -875,7 +885,19 @@ HgfsTruncatePages(struct inode *inode, // IN: Inode whose page to truncate
    ASSERT(inode);
 
    LOG(4, (KERN_DEBUG "VMware hgfs: HgfsTruncatePages: entered\n"));
-   result = compat_vmtruncate(inode, newSize);
+
+   /*
+    * In 3.8.0, vmtruncate was removed and replaced by calling the check
+    * size and set directly.
+    */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 8, 0)
+   result = vmtruncate(inode, newSize);
+#else
+   result = inode_newsize_ok(inode, newSize);
+   if (0 == result) {
+      truncate_setsize(inode, newSize);
+   }
+#endif
    if (result) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsTruncatePages: vmtruncate failed "
               "with error code %d\n", result));
@@ -952,7 +974,12 @@ static int
 HgfsCreate(struct inode *dir,     // IN: Parent dir to create in
            struct dentry *dentry, // IN: Dentry containing name to create
            compat_umode_t mode,   // IN: Mode of file to be created
-	   struct nameidata *nd)  // IN: Intent, vfsmount, ...
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
+           bool excl              // IN: O_EXCL
+#else
+           struct nameidata *nd   // IN: Intent, vfsmount, ...
+#endif
+           )
 {
    HgfsAttrInfo attr;
    int result;
@@ -1025,7 +1052,12 @@ HgfsCreate(struct inode *dir,     // IN: Parent dir to create in
 static struct dentry *
 HgfsLookup(struct inode *dir,      // IN: Inode of parent directory
            struct dentry *dentry,  // IN: Dentry containing name to look up
-           struct nameidata *nd)   // IN: Intent, vfsmount, ...
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
+           unsigned int flags
+#else
+           struct nameidata *nd    // IN: Intent, vfsmount, ...
+#endif
+           )
 {
    HgfsAttrInfo attr;
    struct inode *inode;
@@ -1751,7 +1783,7 @@ HgfsAccessInt(struct dentry *dentry, // IN: dentry to check access for
       }
 
       if ((effectivePermissions & mask) != mask) {
-         ret = -EPERM;
+         ret = -EACCES;
       }
       LOG(8, ("VMware Hgfs: %s: effectivePermissions: %d, ret: %d\n",
               __func__, effectivePermissions, ret));
@@ -1779,13 +1811,50 @@ HgfsAccessInt(struct dentry *dentry, // IN: dentry to check access for
  *----------------------------------------------------------------------
  */
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3, 6, 0)
+static int
+HgfsPermission(struct inode *inode,
+               int mask)
+{
+   LOG(8, ("VMware hgfs: %s: inode->mode: %8x mask: %8x\n", __func__,
+           inode->i_mode, mask));
+   /*
+    * For sys_access, we go to the host for permission checking;
+    * otherwise return 0.
+    */
+   if (mask & MAY_ACCESS) { /* For sys_access. */
+      struct dentry *dentry;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 9, 0)
+      struct hlist_node *p;
+#endif
+
+      if (mask & MAY_NOT_BLOCK)
+         return -ECHILD;
+
+      /* Find a dentry with valid d_count. Refer bug 587879. */
+      hlist_for_each_entry(dentry,
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 9, 0)
+                           p,
+#endif
+                           &inode->i_dentry,
+                           d_alias) {
+         int dcount = dentry->d_count;
+         if (dcount) {
+            LOG(4, ("Found %s %d \n", dentry->d_name.name, dcount));
+            return HgfsAccessInt(dentry, mask & (MAY_READ | MAY_WRITE | MAY_EXEC));
+         }
+      }
+      ASSERT(FALSE);
+   }
+   return 0;
+}
+#else
 static int
 HgfsPermission(struct inode *inode,
                int mask
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
                , struct nameidata *nd
-#endif
-#ifdef IPERM_FLAG_RCU /* introduced in 2.6.38 */
+#elif defined(IPERM_FLAG_RCU)
                , unsigned int flags
 #endif
                )
@@ -1802,23 +1871,26 @@ HgfsPermission(struct inode *inode,
    if (mask & MAY_ACCESS) { /* For sys_access. */
 #endif
       struct list_head *pos;
-      int dcount = 0;
-      struct dentry *dentry = NULL;
 
-#ifdef IPERM_FLAG_RCU
       /*
        * In 2.6.38 path walk is done in 2 distinct modes: rcu-walk and
        * ref-walk. Ref-walk is the classic one; rcu is lockless and is
        * not allowed to sleep. We insist on using ref-walk since our
-       * transports may sleep.
+       * transports may sleep. In 3.1 IPERM_FLAG_RCU was replaced with
+       * MAY_NOT_BLOCK.
        */
+#if defined(MAY_NOT_BLOCK)
+      if (mask & MAY_NOT_BLOCK)
+         return -ECHILD;
+#elif defined(IPERM_FLAG_RCU)
       if (flags & IPERM_FLAG_RCU)
          return -ECHILD;
 #endif
 
-      /* Find a dentry with valid d_count. Refer bug 587789. */
+      /* Find a dentry with valid d_count. Refer bug 587879. */
       list_for_each(pos, &inode->i_dentry) {
-         dentry = list_entry(pos, struct dentry, d_alias);
+         int dcount;
+         struct dentry *dentry = list_entry(pos, struct dentry, d_alias);
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 38)
          dcount = atomic_read(&dentry->d_count);
 #else
@@ -1826,14 +1898,14 @@ HgfsPermission(struct inode *inode,
 #endif
          if (dcount) {
             LOG(4, ("Found %s %d \n", (dentry)->d_name.name, dcount));
-            break;
+            return HgfsAccessInt(dentry, mask & (MAY_READ | MAY_WRITE | MAY_EXEC));
          }
       }
-      ASSERT(dcount);
-      return HgfsAccessInt(dentry, mask & (MAY_READ | MAY_WRITE | MAY_EXEC));
+      ASSERT(FALSE);
    }
    return 0;
 }
+#endif
 
 
 /*
