@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009-2011 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -110,7 +110,7 @@ VSockVmciNotifyWaitingWrite(VSockVmciSock *vsk)    // IN
     *   if writeNotifyWindow > bufferReady then notify
     * as freeSpace == ConsumeSize - bufferReady.
     */
-   retval = VMCIQPair_ConsumeFreeSpace(vsk->qpair) > notifyLimit;
+   retval = vmci_qpair_consume_free_space(vsk->qpair) > notifyLimit;
 #ifdef VSOCK_OPTIMIZATION_FLOW_CONTROL
    if (retval) {
       /*
@@ -158,7 +158,7 @@ VSockVmciNotifyWaitingRead(VSockVmciSock *vsk)  // IN
     * not require a protocol change and will retain compatibility between
     * endpoints with mixed versions of this function.
     */
-   return VMCIQPair_ProduceBufReady(vsk->qpair) > 0;
+   return vmci_qpair_produce_buf_ready(vsk->qpair) > 0;
 #else
    return TRUE;
 #endif
@@ -342,7 +342,7 @@ VSockVmciSendWaitingRead(struct sock *sk,    // IN
              vsk->consumeSize);
    }
 
-   VMCIQPair_GetConsumeIndexes(vsk->qpair, &tail, &head);
+   vmci_qpair_get_consume_indexes(vsk->qpair, &tail, &head);
    roomLeft = vsk->consumeSize - head;
    if (roomNeeded >= roomLeft) {
       waitingInfo.offset = roomNeeded - roomLeft;
@@ -401,7 +401,7 @@ VSockVmciSendWaitingWrite(struct sock *sk,   // IN
       return TRUE;
    }
 
-   VMCIQPair_GetProduceIndexes(vsk->qpair, &tail, &head);
+   vmci_qpair_get_produce_indexes(vsk->qpair, &tail, &head);
    roomLeft = vsk->produceSize - tail;
    if (roomNeeded + 1 >= roomLeft) {
       /* Wraps around to current generation. */
@@ -603,7 +603,7 @@ VSockVmciNotifyPktSocketDestruct(struct sock *sk) // IN
 
 static int32
 VSockVmciNotifyPktPollIn(struct sock *sk,    // IN
-                         int target,         // IN
+                         size_t target,      // IN
                          Bool *dataReadyNow) // IN
 {
    VSockVmciSock *vsk;
@@ -652,7 +652,7 @@ VSockVmciNotifyPktPollIn(struct sock *sk,    // IN
 
 static int32
 VSockVmciNotifyPktPollOut(struct sock *sk,     // IN
-                          int target,          // IN
+                          size_t target,       // IN
                           Bool *spaceAvailNow) // IN
 {
    int64 produceQFreeSpace;
@@ -705,7 +705,7 @@ VSockVmciNotifyPktPollOut(struct sock *sk,     // IN
 
 static int32
 VSockVmciNotifyPktRecvInit(struct sock *sk,               // IN
-                           int target,                    // IN
+                           size_t target,                 // IN
                            VSockVmciRecvNotifyData *data) // IN
 {
    VSockVmciSock *vsk;
@@ -769,7 +769,7 @@ VSockVmciNotifyPktRecvInit(struct sock *sk,               // IN
 
 static int32
 VSockVmciNotifyPktRecvPreBlock(struct sock *sk,               // IN
-                               int target,                    // IN
+                               size_t target,                 // IN
                                VSockVmciRecvNotifyData *data) // IN
 {
    int err;
@@ -817,7 +817,7 @@ VSockVmciNotifyPktRecvPreBlock(struct sock *sk,               // IN
 
 static int32
 VSockVmciNotifyPktRecvPreDequeue(struct sock *sk,               // IN
-                                 int target,                    // IN
+                                 size_t target,                 // IN
                                  VSockVmciRecvNotifyData *data) // IN
 {
    VSockVmciSock *vsk;
@@ -832,9 +832,9 @@ VSockVmciNotifyPktRecvPreDequeue(struct sock *sk,               // IN
     * socket locked we should copy at least ready bytes.
     */
 #if defined(VSOCK_OPTIMIZATION_WAITING_NOTIFY)
-   VMCIQPair_GetConsumeIndexes(vsk->qpair,
-                               &data->produceTail,
-                               &data->consumeHead);
+   vmci_qpair_get_consume_indexes(vsk->qpair,
+                                  &data->produceTail,
+                                  &data->consumeHead);
 #endif
 
    return 0;
@@ -859,7 +859,7 @@ VSockVmciNotifyPktRecvPreDequeue(struct sock *sk,               // IN
 
 static int32
 VSockVmciNotifyPktRecvPostDequeue(struct sock *sk,               // IN
-                                  int target,                    // IN
+                                  size_t target,                 // IN
                                   ssize_t copied,                // IN
                                   Bool dataRead,                 // IN
                                   VSockVmciRecvNotifyData *data) // IN
@@ -991,9 +991,9 @@ VSockVmciNotifyPktSendPreEnqueue(struct sock *sk,               // IN
    vsk = vsock_sk(sk);
 
 #if defined(VSOCK_OPTIMIZATION_WAITING_NOTIFY)
-      VMCIQPair_GetProduceIndexes(vsk->qpair,
-                                  &data->produceTail,
-                                  &data->consumeHead);
+      vmci_qpair_get_produce_indexes(vsk->qpair,
+                                     &data->produceTail,
+                                     &data->consumeHead);
 #endif
 
    return 0;;
