@@ -54,6 +54,14 @@
 #ifdef sun
 #define ACCESSPERMS           (S_IRWXU | S_IRWXG | S_IRWXO)
 #endif
+#ifdef __ANDROID__
+/*
+ * Android doesn't support setmntent(), endmntent() or MOUNTED.
+ */
+#define NO_SETMNTENT
+#define NO_ENDMNTENT
+#define ACCESSPERMS           (S_IRWXU | S_IRWXG | S_IRWXO)
+#endif
 
 
 /*
@@ -504,8 +512,11 @@ DnD_CheckBlockFuse(int blockFd)                    // IN
    }
 
    if (size != sizeof(VMBLOCK_FUSE_READ_RESPONSE)) {
-      LOG(4, ("%s: Response too short (%"FMTSZ"d vs. %"FMTSZ"u).\n",
-              __func__, size, sizeof(VMBLOCK_FUSE_READ_RESPONSE)));
+      /*
+       * Refer to bug 817761 of casting size to size_t.
+       */
+      LOG(4, ("%s: Response too short (%"FMTSZ"u vs. %"FMTSZ"u).\n",
+              __func__, (size_t)size, sizeof(VMBLOCK_FUSE_READ_RESPONSE)));
 
       return FALSE;
    }
@@ -546,17 +557,36 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
                    mode_t vmbDeviceMode,           // IN
                    Bool (*verifyBlock)(int fd))    // IN
 {
+#if defined NO_SETMNTENT || defined NO_ENDMNTENT
+   NOT_IMPLEMENTED();
+   errno = ENOSYS;
+   return -1;
+#else
    Bool found = FALSE;
    int blockFd = -1;
+   char *realMntPoint;
    MNTHANDLE fp;
    DECLARE_MNTINFO(mnt);
 
+   ASSERT(vmbFsName);
+   ASSERT(vmbMntPoint);
+   ASSERT(vmbDevice);
+
+   /* Resolve desired mount point in case it is symlinked somewhere */
+   realMntPoint = Posix_RealPath(vmbMntPoint);
+   if (!realMntPoint) {
+      /*
+       * If resolve failed for some reason try to fall back to
+       * original mount point specification.
+       */
+      realMntPoint = Util_SafeStrdup(vmbMntPoint);
+   }
+
    /* Make sure the vmblock file system is mounted. */
    fp = OPEN_MNTFILE("r");
    if (fp == NULL) {
       LOG(1, ("%s: could not open mount file\n", __func__));
-
-      return -1;
+      goto out;
    }
 
    while (GETNEXT_MNTINFO(fp, mnt)) {
@@ -566,7 +596,7 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
        */
 
       if (strcmp(MNTINFO_FSTYPE(mnt), vmbFsName) == 0 &&
-          strcmp(MNTINFO_MNTPT(mnt), vmbMntPoint) == 0) {
+          strcmp(MNTINFO_MNTPT(mnt), realMntPoint) == 0) {
          found = TRUE;
          break;
       }
@@ -592,7 +622,10 @@ DnD_TryInitVmblock(const char *vmbFsName,          // IN
       }
    }
 
+out:
+   free(realMntPoint);
    return blockFd;
+#endif
 }
 
 
