@@ -77,7 +77,7 @@ ToolsCoreSigHUPCb(const siginfo_t *info,
  * @return FALSE
  */
 
-gboolean
+static gboolean
 ToolsCoreSigHandler(const siginfo_t *info,
                     gpointer data)
 {
@@ -95,7 +95,7 @@ ToolsCoreSigHandler(const siginfo_t *info,
  * @return TRUE
  */
 
-gboolean
+static gboolean
 ToolsCoreSigUsrHandler(const siginfo_t *info,
                        gpointer data)
 {
@@ -104,6 +104,43 @@ ToolsCoreSigUsrHandler(const siginfo_t *info,
 }
 
 
+/**
+ * Perform (optional) work before or after running the main loop.
+ *
+ * @param[in]  state    Service state.
+ * @param[in]  before   TRUE if before running the main loop, FALSE if after.
+ */
+
+static void
+ToolsCoreWorkAroundLoop(ToolsServiceState *state,
+                        gboolean before)
+{
+#ifdef __APPLE__
+   if (state->mainService) {
+      char *libDir = GuestApp_GetInstallPath();
+      char *argv[] = {
+         NULL,
+         before ? "--startInternal" : "--stopInternal",
+         NULL,
+      };
+
+      if (!libDir) {
+         g_error("Failed to retrieve libDir.\n");
+      }
+
+      argv[0] = g_strdup_printf("%s/services.sh", libDir);
+      free(libDir);
+      if (!argv[0]) {
+         g_error("Failed to construct argv[0].\n");
+      }
+
+      g_spawn_sync(NULL, argv, NULL, 0, NULL, NULL, NULL, NULL, NULL, NULL);
+      free(argv[0]);
+   }
+#endif
+}
+
+
 /**
  * Tools daemon entry function.
  *
@@ -211,6 +248,7 @@ main(int argc,
                             ToolsCoreSigHandler, gState.ctx.mainLoop, NULL);
    g_source_unref(src);
 
+   /* On Mac OS, launchd uses SIGTERM. */
    src = VMTools_NewSignalSource(SIGTERM);
    VMTOOLSAPP_ATTACH_SOURCE(&gState.ctx, src,
                             ToolsCoreSigHandler, gState.ctx.mainLoop, NULL);
@@ -230,7 +268,9 @@ main(int argc,
     */
    gState.ctx.envp = System_GetNativeEnviron(envp);
 
+   ToolsCoreWorkAroundLoop(&gState, TRUE);
    ret = ToolsCore_Run(&gState);
+   ToolsCoreWorkAroundLoop(&gState, FALSE);
 
    if (gState.pidFile != NULL) {
       g_unlink(gState.pidFile);
@@ -238,4 +278,3 @@ main(int argc,
 exit:
    return ret;
 }
-
