@@ -32,9 +32,15 @@
 #include "str.h"
 #include "strutil.h"
 #include "toolsCoreInt.h"
-#include "vmtools.h"
 #include "vm_tools_version.h"
+#include "vmware/tools/utils.h"
 
+#if defined(__linux__)
+#include <sys/io.h>
+#include <errno.h>
+#include <string.h>
+#include "ioplGet.h"
+#endif
 
 /**
  * Take action after an RPC channel reset.
@@ -58,7 +64,9 @@ ToolsCoreCheckReset(struct RpcChannel *chan,
       gchar *msg;
 
       app = ToolsCore_GetTcloName(state);
-      g_assert(app != NULL);
+      if (app == NULL) {
+         app = state->name;
+      }
 
       msg = g_strdup_printf("vmx.capability.unified_loop %s", app);
       if (!RpcChannel_Send(state->ctx.rpc, msg, strlen(msg) + 1, NULL, NULL)) {
@@ -71,9 +79,7 @@ ToolsCoreCheckReset(struct RpcChannel *chan,
        * Log the Tools build number to the VMX log file. We don't really care
        * if sending the message fails.
        */
-      msg = g_strdup_printf("log %s: Version: %s",
-                            ToolsCore_GetTcloName(state),
-                            BUILD_NUMBER);
+      msg = g_strdup_printf("log %s: Version: %s", app, BUILD_NUMBER);
       RpcChannel_Send(state->ctx.rpc, msg, strlen(msg) + 1, NULL, NULL);
       g_free(msg);
 
@@ -96,7 +102,7 @@ ToolsCoreCheckReset(struct RpcChannel *chan,
  * @return TRUE.
  */
 
-static Bool
+static gboolean
 ToolsCoreRpcCapReg(RpcInData *data)
 {
    char *confPath = GuestApp_GetConfPath();
@@ -120,7 +126,7 @@ ToolsCoreRpcCapReg(RpcInData *data)
    if (!RpcChannel_Send(state->ctx.rpc, msg, strlen(msg) + 1, NULL, NULL)) {
       g_debug("Unable to register guest conf directory capability.\n");
    }
-   free(msg);
+   g_free(msg);
    msg = NULL;
 
    /* Send the tools version to the VMX. */
@@ -152,6 +158,38 @@ ToolsCoreRpcCapReg(RpcInData *data)
       g_free(toolsVersion);
    }
 
+#if defined (__linux__)
+   /* Send the IOPL elevation capability to VMX. */
+   if (state->mainService) {
+      unsigned int oldLevel;
+      char *result = NULL;
+      size_t resultLen;
+
+      const char ioplElev[] = "tools.capability.iopl_elevation";
+
+      oldLevel = Iopl_Get();
+      g_debug("%s: old IOPL = %u\n", __FUNCTION__, oldLevel);
+
+      if (iopl(3) < 0) {
+         g_debug("Error raising the IOPL, %s", strerror(errno));
+      }
+
+      g_debug("%s: new IOPL = %u\n", __FUNCTION__, Iopl_Get());
+
+      if (!RpcChannel_Send(state->ctx.rpc, ioplElev, sizeof ioplElev,
+                           &result, &resultLen)) {
+         g_debug("Error setting tools iopl elevation capability: %s\n",
+                 result);
+         vm_free(result);
+      }
+
+      if (iopl(oldLevel) < 0) {
+         g_debug("Error restoring the IOPL, %s", strerror(errno));
+      }
+   }
+#endif
+
+   state->capsRegistered = TRUE;
    free(confPath);
    return RPCIN_SETRETVALS(data, "", TRUE);
 }
@@ -163,10 +201,10 @@ ToolsCoreRpcCapReg(RpcInData *data)
  *
  * @param[in]  data     The RPC data.
  *
- * @return TRUE.
+ * @return Whether the option was successfully processed.
  */
 
-static Bool
+static gboolean
 ToolsCoreRpcSetOption(RpcInData *data)
 {
 
@@ -182,29 +220,23 @@ ToolsCoreRpcSetOption(RpcInData *data)
    index++;
    value = StrUtil_GetNextToken(&index, data->args, "");
 
-   if (option == NULL || value == NULL || strlen(value) == 0) {
-      goto exit;
+   if (option != NULL && value != NULL && strlen(value) != 0) {
+
+      g_debug("Setting option '%s' to '%s'.\n", option, value);
+      g_signal_emit_by_name(state->ctx.serviceObj,
+                            TOOLS_CORE_SIG_SET_OPTION,
+                            &state->ctx,
+                            option,
+                            value,
+                            &retVal);
    }
 
-   g_debug("Setting option '%s' to '%s'.\n", option, value);
-   g_key_file_set_string(state->ctx.config, state->ctx.name, option, value);
-
-   g_signal_emit_by_name(state->ctx.serviceObj,
-                         TOOLS_CORE_SIG_SET_OPTION,
-                         &state->ctx,
-                         option,
-                         value,
-                         &retVal);
-
-exit:
    vm_free(option);
    vm_free(value);
-   if (retVal) {
-      RPCIN_SETRETVALS(data, "", retVal);
-   } else {
-      RPCIN_SETRETVALS(data, "Unknown or invalid option", retVal);
-   }
-   return (Bool) retVal;
+
+   RPCIN_SETRETVALS(data, retVal ? "" : "Unknown or invalid option", retVal);
+
+   return retVal;
 }
 
 
@@ -229,7 +261,7 @@ ToolsCore_InitRpc(ToolsServiceState *state)
    const gchar *app;
    GMainContext *mainCtx = g_main_loop_get_context(state->ctx.mainLoop);
 
-   g_assert(state->ctx.rpc == NULL);
+   ASSERT(state->ctx.rpc == NULL);
 
    if (state->debugPlugin != NULL) {
       app = "debug";
@@ -250,10 +282,13 @@ ToolsCore_InitRpc(ToolsServiceState *state)
                    state->name);
          state->ctx.rpc = NULL;
       } else {
-         state->ctx.rpc = RpcChannel_NewBackdoorChannel(mainCtx);
+         state->ctx.rpc = BackdoorChannel_New();
       }
       app = ToolsCore_GetTcloName(state);
-      g_assert(app != NULL);
+      if (app == NULL) {
+         g_warning("Trying to start RPC channel for invalid %s container.", state->name);
+         return FALSE;
+      }
    }
 
    if (state->ctx.rpc) {
