@@ -30,6 +30,7 @@
 #   include <signal.h>
 #endif
 
+#include "vm_assert.h"
 #include "toolboxCmdInt.h"
 #include "guestApp.h"
 #include "wiper.h"
@@ -90,12 +91,13 @@ typedef enum {
  */
 
 static WiperState
-ShrinkGetWiperState(void) {
-   char *result;
+ShrinkGetWiperState(void)
+{
+   char *result = NULL;
    size_t resultLen;
    WiperState state = WIPER_UNAVAILABLE;
 
-   if (ToolsCmd_SendRPC(WIPER_STATE_CMD, sizeof WIPER_STATE_CMD,
+   if (ToolsCmd_SendRPC(WIPER_STATE_CMD, sizeof WIPER_STATE_CMD - 1,
                         &result, &resultLen)) {
       if (resultLen == 1 && strcmp(result, "1") == 0) {
          state = WIPER_ENABLED;
@@ -127,66 +129,37 @@ ShrinkGetWiperState(void) {
 static Bool
 ShrinkGetMountPoints(WiperPartition_List *pl) // OUT: Known mount points
 {
-   if (ShrinkGetWiperState() != WIPER_UNAVAILABLE) {
+   WiperState state = ShrinkGetWiperState();
+
+   switch (state) {
+   case WIPER_UNAVAILABLE:
       ToolsCmd_PrintErr("%s",
                         SU_(disk.shrink.unavailable, SHRINK_FEATURE_ERR));
-   } else if (!WiperPartition_Open(pl)) {
+      break;
+
+   case WIPER_DISABLED:
+      ToolsCmd_PrintErr("%s",
+                        SU_(disk.shrink.disabled, SHRINK_DISABLED_ERR));
+      break;
+
+   case WIPER_ENABLED:
+      if (WiperPartition_Open(pl)) {
+         return TRUE;
+      }
+
       ToolsCmd_PrintErr("%s",
                         SU_(disk.shrink.partition.error,
                            "Unable to collect partition data.\n"));
-   } else {
-      return TRUE;
+      break;
+
+   default:
+      NOT_REACHED();
    }
+
    return FALSE;
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * ShrinkGetPartition  --
- *
- *      Finds the WiperPartion whose mountpoint is given.
- *
- * Results:
- *      The WiperPatition.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-static WiperPartition*
-ShrinkGetPartition(char *mountPoint)
-{
-   WiperPartition_List plist;
-   WiperPartition *p, *part = NULL;
-   DblLnkLst_Links *curr;
-
-   if (!ShrinkGetMountPoints(&plist)) {
-      return NULL;
-   }
-
-   DblLnkLst_ForEach(curr, &plist.link) {
-      p = DblLnkLst_Container(curr, WiperPartition, link);
-      if (toolbox_strcmp(p->mountPoint, mountPoint) == 0) {
-         part = p;
-         /*
-          * Detach the element we are interested in so it is not
-          * destroyed when we call WiperPartition_Close.
-          */
-         DblLnkLst_Unlink1(&part->link);
-         break;
-      }
-   }
-
-   WiperPartition_Close(&plist);
-
-   return part;
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -209,6 +182,7 @@ ShrinkList(void)
    WiperPartition_List plist;
    DblLnkLst_Links *curr;
    uint32 countShrink = 0;
+   WiperState wstate = ShrinkGetWiperState();
 
    if (!ShrinkGetMountPoints(&plist)) {
       return EX_TEMPFAIL;
@@ -217,7 +191,7 @@ ShrinkList(void)
    DblLnkLst_ForEach(curr, &plist.link) {
       WiperPartition *p = DblLnkLst_Container(curr, WiperPartition, link);
       if (p->type != PARTITION_UNSUPPORTED &&
-          (ShrinkGetWiperState() == WIPER_ENABLED || Wiper_IsWipeSupported(p))) {
+          (wstate == WIPER_ENABLED || Wiper_IsWipeSupported(p))) {
          g_print("%s\n", p->mountPoint);
          countShrink++;
       }
@@ -302,6 +276,7 @@ ShrinkDoAllDiskShrinkOnly(void)
    WiperPartition_List plist;
    DblLnkLst_Links *curr;
    Bool canShrink = FALSE;
+   WiperState wstate = ShrinkGetWiperState();
 
 #ifndef _WIN32
    signal(SIGINT, ShrinkWiperDestroy);
@@ -314,7 +289,7 @@ ShrinkDoAllDiskShrinkOnly(void)
    DblLnkLst_ForEach(curr, &plist.link) {
       WiperPartition *p = DblLnkLst_Container(curr, WiperPartition, link);
       if (p->type != PARTITION_UNSUPPORTED &&
-          (ShrinkGetWiperState() == WIPER_ENABLED || Wiper_IsWipeSupported(p))) {
+          (wstate == WIPER_ENABLED || Wiper_IsWipeSupported(p))) {
          canShrink = TRUE;
          break;
       }
@@ -367,14 +342,36 @@ ShrinkDoWipeAndShrink(char *mountPoint,         // IN: mount point
    int i;
    int progress = 0;
    unsigned char *err;
-   WiperPartition *part;
+   WiperPartition *part = NULL;
+   WiperPartition_List plist;
    int rc;
 
-#ifndef _WIN32
+#if defined(_WIN32)
+   DWORD currPriority = GetPriorityClass(GetCurrentProcess());
+#else
    signal(SIGINT, ShrinkWiperDestroy);
 #endif
 
-   part = ShrinkGetPartition(mountPoint);
+   if (ShrinkGetMountPoints(&plist)) {
+      DblLnkLst_Links *curr, *nextElem;
+      DblLnkLst_ForEachSafe(curr, nextElem, &plist.link) {
+         WiperPartition *p = DblLnkLst_Container(curr, WiperPartition, link);
+         if (toolbox_strcmp(p->mountPoint, mountPoint) == 0) {
+            WiperSinglePartition_Close(part);
+            part = p;
+            /*
+             * Detach the element we are interested in so it is not
+             * destroyed when we call WiperPartition_Close.
+             */
+            DblLnkLst_Unlink1(&part->link);
+            if (part->type != PARTITION_UNSUPPORTED) {
+               break;
+            }
+         }
+      }
+      WiperPartition_Close(&plist);
+   }
+
    if (part == NULL) {
       ToolsCmd_PrintErr(SU_(disk.shrink.partition.notfound,
                             "Unable to find partition %s\n"),
@@ -402,8 +399,35 @@ ShrinkDoWipeAndShrink(char *mountPoint,         // IN: mount point
       goto out;
    }
 
+   /*
+    * During the initial 'wipe' process, the Toolbox CLI first fills the
+    * entire guest's disk space with files filled with zeroes. During this step,
+    * user may notice few warning messages related to 'low disk space' in the
+    * guest operating system. We need to print a warning message to disregard
+    * such warnings in the guest operating system.
+    */
+   if (performShrink) {
+      ToolsCmd_Print("%s", SU_(disk.shrink.ignoreFreeSpaceWarnings,
+                               "Please disregard any warnings about disk space "
+                               "for the duration of shrink process.\n"));
+   } else {
+      ToolsCmd_Print("%s", SU_(disk.wipe.ignoreFreeSpaceWarnings,
+                               "Please disregard any warnings about disk space "
+                               "for the duration of wipe process.\n"));
+   }
+
    wiper = Wiper_Start(part, MAX_WIPER_FILE_SIZE);
 
+#if defined(_WIN32)
+   /*
+    * On Win32, lower the process priority during wipe, so other applications
+    * can still run (sort of) normally while we're filling the disk.
+    */
+   if (!SetPriorityClass(GetCurrentProcess(), BELOW_NORMAL_PRIORITY_CLASS)) {
+      g_debug("Unable to lower process priority: %u.", GetLastError());
+   }
+#endif
+
    while (progress < 100 && wiper != NULL) {
       err = Wiper_Next(&wiper, &progress);
       if (strlen(err) > 0) {
@@ -429,6 +453,13 @@ ShrinkDoWipeAndShrink(char *mountPoint,         // IN: mount point
       }
    }
 
+#if defined(_WIN32)
+   /* Go back to our original priority. */
+   if (!SetPriorityClass(GetCurrentProcess(), currPriority)) {
+      g_debug("Unable to restore process priority: %u.", GetLastError());
+   }
+#endif
+
    rc = EXIT_SUCCESS;
    if (progress >= 100 && performShrink) {
       rc = ShrinkDiskSendRPC();
